@@ -24,14 +24,14 @@ use DBD::SQLite   ();
 # TO DO: Bug report dispatched. Likely to be fixed in 0.77.
 use version ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 
 # Since everything is used OO-style, we will be require'ing
 # everything other than the bare essentials
 use Padre::Constant ();
 use Padre::Config   ();
 use Padre::DB       ();
-use Padre::Action::Queue;
+use Padre::Logger;
 
 # Generate faster accessors
 use Class::XSAccessor 1.05 {
@@ -45,10 +45,13 @@ use Class::XSAccessor 1.05 {
 	},
 	accessors => {
 		actions     => 'actions',
+		wizards     => 'wizards',
+		shortcuts   => 'shortcuts',
 		instance_id => 'instance_id',
 	},
 };
 
+
 sub import {
 	unless ( $_[1] and $_[1] eq ':everything' ) {
 		return;
@@ -134,9 +137,12 @@ sub new {
 	# Load (and sync if needed) the configuration
 	$self->{config} = Padre::Config->read;
 
-	# Actions registry
-	my %actions = ();
-	$self->actions( \%actions );
+	# Actions and keyboard shortcuts registries
+	$self->actions(   {} );
+	$self->shortcuts( {} );
+
+	# Wizard registry
+	$self->wizards( {} );
 
 	# Load a few more bits and pieces now we know
 	# that we'll need them
@@ -153,12 +159,10 @@ sub new {
 	# Create the task manager
 	require Padre::TaskManager;
 	$self->{task_manager} = Padre::TaskManager->new(
-		use_threads => $self->config->threads,
+		threads => 1,
+		conduit => $self->{wx}->{main},
 	);
 
-	# Create the action queue
-	$self->{actionqueue} = Padre::Action::Queue->new;
-
 	# Startup completed, let go of the database
 	Padre::DB->commit;
 
@@ -171,6 +175,8 @@ sub run {
 	# Allow scripts to detect that they are being executed within Padre
 	local $ENV{PADRE_VERSION} = $VERSION;
 
+	TRACE("Padre->run was called version $VERSION") if DEBUG;
+
 	# Clean arguments (with a bad patch for saving URLs)
 	if (Padre::Constant::WIN32) {
 
@@ -198,6 +204,8 @@ sub run {
 		$self->plugin_manager->load_plugins;
 	}
 
+	TRACE("Plugins loaded") if DEBUG;
+
 	# Move our current dir to the user's documents directory by default
 	if (Padre::Constant::WIN32) {
 
@@ -209,22 +217,16 @@ sub run {
 		}
 	}
 
-	# Check if we have Time::HiRes:
-	# This should be better done in a background job
-	if ( eval { require Time::HiRes; } and ( !$@ ) ) {
-		$self->{has_Time_HiRes} = 1;
-	}
-
 	# HACK: Uncomment this to locate difficult-to-find crashes
 	#       that are throw silent exceptions.
 	# local $SIG{__DIE__} = sub { print @_; die $_[0] };
 
-	# Kill the splash screen
+	TRACE("Kill the splash screen") if DEBUG;
 	if ($Padre::Startup::VERSION) {
 		Padre::Startup->destroy_splash;
 	}
 
-	# Process the action queue
+	TRACE("Process the action queue") if DEBUG;
 	if ( defined $self->opts->{actionqueue} ) {
 		foreach my $action ( split( /\,/, $self->opts->{actionqueue} ) ) {
 			next if $action eq ''; # Skip empty action names
@@ -234,11 +236,11 @@ sub run {
 			}
 
 			# Add the action to the queue
-			$self->{actionqueue}->add($action);
+			$self->wx->queue->add($action);
 		}
 	}
 
-	# Switch into runtime mode
+	TRACE("Switch into runtime mode") if DEBUG;
 	$self->wx->MainLoop;
 
 	# All shutdown procedures complete.
@@ -270,6 +272,10 @@ sub project {
 	return $self->{project}->{$root};
 }
 
+sub project_exists {
+	defined $_[0]->{project}->{ $_[1] };
+}
+
 1;
 
 __END__
@@ -970,12 +976,6 @@ know about Wx, but currently it still does.
 
 =item L<Padre::Wx>
 
-=item L<Padre::Wx::Ack>
-
-Implementation of the L<ack> integration in C<Edit/Ack> menu item.
-It probably should be either under Dialog or moved out to be a
-plug-in.
-
 =item L<Padre::Wx::App>
 
 is the L<Wx::App> subclass. Does not really do much.
@@ -1117,6 +1117,10 @@ Ahmad Zawawi أحمد محمد زواوي (AZAWAWI)
 
 Adam Kennedy (ADAMK) E<lt>adamk@cpan.orgE<gt>
 
+Alexandr Ciornii (CHORNY)
+
+Blake Willmarth (BLAKEW)
+
 Breno G. de Oliveira (GARU)
 
 Brian Cassidy (BRICAS)
@@ -1163,7 +1167,7 @@ Sebastian Willing (SEWI)
 
 Steffen Müller (TSEE) E<lt>smueller@cpan.orgE<gt>
 
-Zeno Gantner
+Zeno Gantner (ZENOG)
 
 =head2 Translators
 
@@ -1201,7 +1205,7 @@ Heiko Jansen (HJANSEN)
 
 Sebastian Willing (SEWI)
 
-Zeno Gantner
+Zeno Gantner (ZENOG)
 
 =head3 Hebrew
 
