@@ -1,11 +1,9 @@
 package Padre::Config;
 
-#
 # Configuration subsystem for Padre
-#
 
-# To help force the break from the first-generate HASH based configuration
-# over to thdee second-generation method based configuration, initially we
+# To help force the break from the first-generation HASH based configuration
+# over to the second-generation method based configuration, initially we
 # will use an ARRAY-based object, so that all existing code is forcefully
 # broken.
 
@@ -13,36 +11,48 @@ use 5.008;
 use strict;
 use warnings;
 use Carp                   ();
+use YAML::Tiny             ();
 use Params::Util           ();
 use Padre::Constant        ();
 use Padre::Util            ('_T');
-use Padre::Current         ('_CURRENT');
+use Padre::Current         ();
 use Padre::Config::Setting ();
 use Padre::Config::Human   ();
 use Padre::Config::Project ();
 use Padre::Config::Host    ();
 use Padre::Config::Upgrade ();
+use Padre::Logger;
 
-our $VERSION = '0.50';
+our $VERSION = '0.76';
 
-# Master storage of the settings
-our %SETTING = ();
+our ( %SETTING, %DEFAULT, %STARTUP, $REVISION, $SINGLETON );
 
-# A cache for the defaults
-our %DEFAULT = ();
+BEGIN {
 
-# The configuration revision.
-# (Functionally similar to the database revision)
-our $REVISION = 1;
+	# Master storage of the settings
+	%SETTING = ();
 
-# Storage for the default config object
-our $SINGLETON = undef;
+	# A cache for the defaults
+	%DEFAULT = ();
+
+	# A cache for startup.yml settings
+	%STARTUP = ();
+
+	# The configuration revision.
+	# (Functionally similar to the database revision)
+	$REVISION = 1;
+
+	# Storage for the default config object
+	$SINGLETON = undef;
+}
 
 # Accessor generation
-use Class::XSAccessor::Array getters => {
-	host    => Padre::Constant::HOST,
-	human   => Padre::Constant::HUMAN,
-	project => Padre::Constant::PROJECT,
+use Class::XSAccessor::Array {
+	getters => {
+		host    => Padre::Constant::HOST,
+		human   => Padre::Constant::HUMAN,
+		project => Padre::Constant::PROJECT,
+	}
 };
 
 
@@ -55,6 +65,10 @@ use Class::XSAccessor::Array getters => {
 # This section identifies the set of all named configuration entries,
 # and where the configuration system should resolve them to.
 
+sub settings {
+	sort keys %SETTING;
+}
+
 #
 # setting( %params );
 #
@@ -67,36 +81,222 @@ sub setting {
 
 	# Validate the setting
 	my $object = Padre::Config::Setting->new(@_);
-	if ( $SETTING{ $object->{name} } ) {
-		Carp::croak("The $object->{name} setting is already defined");
+	my $name   = $object->{name};
+	if ( $SETTING{$name} ) {
+		Carp::croak("The $name setting is already defined");
 	}
 
 	# Generate the accessor
-	my $code = <<"END_PERL";
-package Padre::Config;
-
-sub $object->{name} {
-	my \$self = shift;
-	if ( exists \$self->[$object->{store}]->{'$object->{name}'} ) {
-		return \$self->[$object->{store}]->{'$object->{name}'};
-	}
-	return \$DEFAULT{'$object->{name}'};
-}
-END_PERL
-
-	# Compile the accessor
-	eval $code; ## no critic
-	if ($@) {
-		Carp::croak("Failed to compile setting $object->{name}");
-	}
+	eval $object->code;
+	Carp::croak("Failed to compile setting $object->{name}: $@") if $@;
 
 	# Save the setting
-	$SETTING{ $object->{name} } = $object;
-	$DEFAULT{ $object->{name} } = $object->{default};
+	$SETTING{$name} = $object;
+	$DEFAULT{$name} = $object->{default};
+	$STARTUP{$name} = 1 if $object->{startup};
 
 	return 1;
 }
 
+
+
+
+
+#####################################################################
+# Constructor and Input/Output
+
+sub new {
+	my $class = shift;
+	my $host  = shift;
+	my $human = shift;
+	unless ( Params::Util::_INSTANCE( $host, 'Padre::Config::Host' ) ) {
+		Carp::croak("Did not provide a host config to Padre::Config->new");
+	}
+	unless ( Params::Util::_INSTANCE( $human, 'Padre::Config::Human' ) ) {
+		Carp::croak("Did not provide a user config to Padre::Config->new");
+	}
+
+	# Create the basic object with the two required elements
+	my $self = bless [ $host, $human, undef ], $class;
+
+	# Add the optional third element
+	if (@_) {
+		my $project = shift;
+		unless ( Params::Util::_INSTANCE( $project, 'Padre::Config::Project' ) ) {
+			Carp::croak("Did not provide a project config to Padre::Config->new");
+		}
+		$self->[Padre::Constant::PROJECT] = $project;
+	}
+
+	return $self;
+}
+
+sub read {
+	my $class = shift;
+
+	unless ($SINGLETON) {
+		TRACE("Loading configuration for $class") if DEBUG;
+
+		# Load the host configuration
+		my $host = Padre::Config::Host->read;
+
+		# Load the user configuration
+		my $human = Padre::Config::Human->read
+			|| Padre::Config::Human->create;
+
+		# Hand off to the constructor
+		$SINGLETON = $class->new( $host, $human );
+
+		Padre::Config::Upgrade::check($SINGLETON);
+	}
+
+	return $SINGLETON;
+}
+
+sub write {
+	TRACE( $_[0] ) if DEBUG;
+	my $self = shift;
+
+	# Save the user configuration
+	$self->[Padre::Constant::HUMAN]->{version} = $REVISION;
+	$self->[Padre::Constant::HUMAN]->write;
+
+	# Save the host configuration
+	$self->[Padre::Constant::HOST]->{version} = $REVISION;
+	$self->[Padre::Constant::HOST]->write;
+
+	# Write the startup subset of the configuration.
+	# NOTE: Use a hyper-minimalist listified key/value file format
+	# so that we don't need to load YAML::Tiny before the thread fork.
+	# This should save around 400k of memory per background thread.
+	my %startup = map { $_ => $self->$_() } sort keys %STARTUP;
+	open( my $FILE, '>', Padre::Constant::CONFIG_STARTUP ) or return 1;
+	print $FILE map {"$_\n$startup{$_}\n"} sort keys %startup or return 1;
+	close $FILE or return 1;
+
+	return 1;
+}
+
+
+
+
+
+######################################################################
+# Main Methods
+
+# Fetches an explicitly named default
+sub default {
+	my $self = shift;
+	my $name = shift;
+
+	# Does the setting exist?
+	unless ( $SETTING{$name} ) {
+		Carp::croak("The configuration setting '$name' does not exist");
+	}
+
+	return $DEFAULT{$name};
+}
+
+sub set {
+	TRACE( $_[1] ) if DEBUG;
+	my $self  = shift;
+	my $name  = shift;
+	my $value = shift;
+
+	# Does the setting exist?
+	my $setting = $SETTING{$name};
+	unless ($setting) {
+		Carp::croak("The configuration setting '$name' does not exist");
+	}
+
+	# All types are Padre::Constant::ASCII-like
+	unless ( defined $value and not ref $value ) {
+		Carp::croak("Missing or non-scalar value for setting '$name'");
+	}
+
+	# We don't need to do additional checks on Padre::Constant::ASCII
+	my $type = $setting->type;
+	if ( $type == Padre::Constant::BOOLEAN ) {
+		$value = 0 if $value eq '';
+		if ( $value ne '1' and $value ne '0' ) {
+			Carp::croak("Tried to change setting '$name' to non-boolean '$value'");
+		}
+	}
+	if ( $type == Padre::Constant::POSINT and not Params::Util::_POSINT($value) ) {
+		Carp::croak("Tried to change setting '$name' to non-posint '$value'");
+	}
+	if ( $type == Padre::Constant::INTEGER and not _INTEGER($value) ) {
+		Carp::croak("Tried to change setting '$name' to non-integer '$value'");
+	}
+	if ( $type == Padre::Constant::PATH ) {
+		if ( Padre::Constant::WIN32 and utf8::is_utf8($value) ) {
+			require Win32;
+			$value = Win32::GetLongPathName($value);
+
+			#Wx::DirPickerCtrl upgrades data to utf8.
+			#Perl on Windows cannot handle utf8 in file names, so this hack converts
+			#path back
+		}
+		if ( not -e $value ) {
+			Carp::croak("Tried to change setting '$name' to non-existant path '$value'");
+		}
+	}
+
+	# Set the value into the appropriate backend
+	my $store = $SETTING{$name}->store;
+	$self->[$store]->{$name} = $value;
+
+	return 1;
+}
+
+# Set a value in the configuration and apply the preference change
+# to the application.
+sub apply {
+	TRACE( $_[0] ) if DEBUG;
+	my $self    = shift;
+	my $name    = shift;
+	my $value   = shift;
+	my $current = Padre::Current::_CURRENT(@_);
+
+	# Set the config value
+	$self->set( $name => $value );
+
+	# Does this setting have an apply hook
+	my $code = $SETTING{$name}->apply;
+	if ($code) {
+		$code->( $current->main, $value );
+	}
+
+	return 1;
+}
+
+sub meta {
+	$SETTING{ $_[1] } or die("Missing or invalid setting name '$_[1]'");
+}
+
+
+
+
+
+######################################################################
+# Support Functions
+
+#
+# my $is_integer = _INTEGER( $scalar );
+#
+# return true if $scalar is an integer.
+#
+sub _INTEGER {
+	return defined $_[0] && !ref $_[0] && $_[0] =~ m/^(?:0|-?[1-9]\d*)$/;
+}
+
+
+
+
+
+######################################################################
+# Basic Settings
+
 # User identity (simplistic initial version)
 # Initially, this must be ascii only
 setting(
@@ -169,37 +369,69 @@ setting(
 	project => 1,
 );
 
+
+
+
+
+#####################################################################
+# Startup Behaviour Rules
+
+setting(
+	name    => 'startup_splash',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+	startup => 1,
+);
+
 # Startup mode, if no files given on the command line this can be
 #   new        - a new empty buffer
 #   nothing    - nothing to open
 #   last       - the files that were open last time
 setting(
-	name    => 'main_startup',
+	name    => 'startup_files',
 	type    => Padre::Constant::ASCII,
 	store   => Padre::Constant::HUMAN,
 	default => 'new',
-	options => [
+	options => {
 		'last'    => _T('Previous open files'),
 		'new'     => _T('A new empty file'),
 		'nothing' => _T('No open files'),
 		'session' => _T('Open session'),
-	],
+	},
 );
 
+# How many times has the user run Padre?
+# Default is 1 and the value is incremented at shutdown rather than
+# startup so that we don't have to write files in the startup sequence.
+setting(
+	name    => 'startup_count',
+	type    => Padre::Constant::POSINT,
+	store   => Padre::Constant::HUMAN,
+	default => 1,
+);
+
+
+
+
+
+######################################################################
+# Main Window Tools and Layout
+
 # Window
 setting(
-	name    => 'window_title',
+	name    => 'main_title',
 	type    => Padre::Constant::ASCII,
 	store   => Padre::Constant::HUMAN,
 	default => 'Padre [%p]',
 );
 
-# Pages and panels
 setting(
 	name    => 'main_singleinstance',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
-	default => 0,
+	default => Padre::Constant::DEFAULT_SINGLEINSTANCE,
+	startup => 1,
 	apply   => sub {
 		my $main  = shift;
 		my $value = shift;
@@ -211,6 +443,15 @@ setting(
 		return 1;
 	},
 );
+
+setting(
+	name    => 'main_singleinstance_port',
+	type    => Padre::Constant::POSINT,
+	store   => Padre::Constant::HOST,
+	default => Padre::Constant::DEFAULT_SINGLEINSTANCE_PORT,
+	startup => 1,
+);
+
 setting(
 	name    => 'main_lockinterface',
 	type    => Padre::Constant::BOOLEAN,
@@ -225,7 +466,7 @@ setting(
 
 		# The toolbar can't dynamically switch between
 		# tearable and non-tearable so rebuild it.
-		# TODO: Review this assumption
+		# TO DO: Review this assumption
 
 		# (Ticket #668)
 
@@ -247,11 +488,11 @@ setting(
 	type    => Padre::Constant::ASCII,
 	store   => Padre::Constant::HUMAN,
 	default => 'alphabetical',
-	options => [
+	options => {
 		'original'                  => _T('Code Order'),
 		'alphabetical'              => _T('Alphabetical Order'),
 		'alphabetical_private_last' => _T('Alphabetical Order (Private Last)'),
-	],
+	},
 );
 setting(
 	name    => 'main_outline',
@@ -259,21 +500,37 @@ setting(
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
+setting(
+	name    => 'main_todo',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
 setting(
 	name    => 'main_directory',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
+setting(
+	name    => 'main_directory_order',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => 'first',
+	options => {
+		first => _T('Directories First'),
+		mixed => _T('Directories Mixed'),
+	},
+);
 setting(
 	name    => 'main_directory_panel',
 	type    => Padre::Constant::ASCII,
 	store   => Padre::Constant::HUMAN,
 	default => 'left',
-	options => [
+	options => {
 		'left'  => _T('Project Tools (Left)'),
 		'right' => _T('Document Tools (Right)'),
-	],
+	},
 	apply => sub {
 		my $main  = shift;
 		my $value = shift;
@@ -293,12 +550,24 @@ setting(
 		return 1;
 	}
 );
+setting(
+	name  => 'main_directory_root',
+	type  => Padre::Constant::ASCII,
+	store => Padre::Constant::HOST,
+	default => File::HomeDir->my_documents || '',
+);
 setting(
 	name    => 'main_output',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
+setting(
+	name    => 'main_command_line',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
 setting(
 	name    => 'main_output_ansi',
 	type    => Padre::Constant::BOOLEAN,
@@ -311,12 +580,6 @@ setting(
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
-setting(
-	name    => 'main_errorlist',
-	type    => Padre::Constant::BOOLEAN,
-	store   => Padre::Constant::HUMAN,
-	default => 0,
-);
 setting(
 	name    => 'main_statusbar',
 	type    => Padre::Constant::BOOLEAN,
@@ -330,6 +593,7 @@ setting(
 
 	# Toolbars are not typically used for Mac apps.
 	# Hide it by default so Padre looks "more Mac'ish"
+	# NOTE: Or at least, so we were told. Opinions apparently vary.
 	default => Padre::Constant::MAC ? 0 : 1,
 );
 setting(
@@ -337,77 +601,74 @@ setting(
 	type  => Padre::Constant::ASCII,
 	store => Padre::Constant::HUMAN,
 
-	# This lifes here until a better place is found:
+	# This lives here until a better place is found:
 	# This is a list of toolbar items, seperated by ;
 	# The following items are supported:
-	#   action:icon
-	#     Insert the action with the named icon
-	#   action(argument,argument):icon
+	#   action
+	#     Insert the action
+	#   action(argument,argument)
 	#     Insert an action which requires one or more arguments
 	#   |
 	#     Insert a seperator
-	default => 'file.new:actions/document-new;'
-		. 'file.open:actions/document-open;'
-		. 'file.save:actions/document-save;'
-		. 'file.save_as:actions/document-save-as;'
-		. 'file.save_all:actions/stock_data-save;'
-		. 'file.close:actions/x-document-close;' . '|;'
-		. 'file.open_example:stock/generic/stock_example;' . '|;'
-		. 'edit.undo:actions/edit-undo;'
-		. 'edit.redo:actions/edit-redo;' . '|;'
-		. 'edit.cut:actions/edit-cut;'
-		. 'edit.copy:actions/edit-copy;'
-		. 'edit.paste:actions/edit-paste;'
-		. 'edit.select_all:actions/edit-select-all;' . '|;'
-		. 'search.find:actions/edit-find;'
-		. 'search.replace:actions/edit-find-replace;' . '|;'
-		. 'edit.comment_toggle:actions/toggle-comments;' . '|;'
-		. 'file.doc_stat:actions/document-properties;' . '|;'
-		. 'search.open_resource:places/folder-saved-search;'
-		. 'search.quick_menu_access:status/info;' . '|;'
-		. 'run.run_document:actions/player_play;'
-		. 'run.stop:actions/stop;',
+	default => 'file.new;'
+		. 'file.open;'
+		. 'file.save;'
+		. 'file.save_as;'
+		. 'file.save_all;'
+		. 'file.close;' . '|;'
+		. 'file.open_example;' . '|;'
+		. 'edit.undo;'
+		. 'edit.redo;' . '|;'
+		. 'edit.cut;'
+		. 'edit.copy;'
+		. 'edit.paste;'
+		. 'edit.select_all;' . '|;'
+		. 'search.find;'
+		. 'search.replace;' . '|;'
+		. 'edit.comment_toggle;' . '|;'
+		. 'file.doc_stat;' . '|;'
+		. 'search.open_resource;'
+		. 'search.quick_menu_access;' . '|;'
+		. 'run.run_document;'
+		. 'run.stop;' . '|;'
+		. 'debug.step_in;'
+		. 'debug.step_over;'
+		. 'debug.step_out;'
+		. 'debug.run;'
+		. 'debug.set_breakpoint;'
+		. 'debug.display_value;'
+		. 'debug.quit;'
 );
 
 setting(
-	name  => 'main_menubar_items',
-	type  => Padre::Constant::ASCII,
-	store => Padre::Constant::HUMAN,
-
-	# This lifes here until a better place is found:
-	# This is a list of menubar items, seperated by ;
-	# The following items are supported:
-	#   menu.MenuName
-	#     Insert the (sub-)menu MenuName
-	#   some.other_action
-	#     Insert an action
-	#   |
-	#     Insert a seperator
-	default => 'menu.File;'
-		. 'menu.Edit;'
-		. 'menu.Search;'
-		. 'menu.View;'
-		. 'menu._document;'
-		. 'menu.Run;'
-		. 'menu.Plugins;'
-		. 'menu.Window;'
-		. 'menu.Help',
+	name    => 'swap_ctrl_tab_alt_right',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
 );
 
 # Directory Tree Settings
 setting(
-	name    => 'default_projects_directory',
-	type    => Padre::Constant::PATH,
-	store   => Padre::Constant::HOST,
+	name  => 'default_projects_directory',
+	type  => Padre::Constant::PATH,
+	store => Padre::Constant::HOST,
 	default => File::HomeDir->my_documents || '',
 );
 
 # Editor Settings
+my $default_editor_font = '';
+if (Padre::Constant::WIN32) {
+
+	# The default editor font should be Consolas 10pt on Vista and Windows 7
+	require Win32;
+	my $os = Win32::GetOSName;
+	$default_editor_font = 'Consolas 10' if $os eq 'WinVista' or $os eq 'Win7';
+}
 setting(
 	name    => 'editor_font',
 	type    => Padre::Constant::ASCII,
 	store   => Padre::Constant::HUMAN,
-	default => '',
+	default => $default_editor_font,
 );
 setting(
 	name    => 'editor_linenumbers',
@@ -444,11 +705,11 @@ setting(
 	type    => Padre::Constant::ASCII,
 	store   => Padre::Constant::HUMAN,
 	default => 'deep',
-	options => [
-		'no'   => 'No Autoindent',
-		'same' => 'Indent to Same Depth',
-		'deep' => 'Indent Deeply',
-	],
+	options => {
+		'no'   => _T('No Autoindent'),
+		'same' => _T('Indent to Same Depth'),
+		'deep' => _T('Indent Deeply'),
+	},
 );
 setting(
 	name    => 'editor_folding',
@@ -462,6 +723,14 @@ setting(
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
+
+setting(
+	name    => 'editor_brace_expression_highlighting',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
+
 setting(
 	name    => 'save_autoclean',
 	type    => Padre::Constant::BOOLEAN,
@@ -516,6 +785,12 @@ setting(
 	store   => Padre::Constant::HUMAN,
 	default => 1,
 );
+setting(
+	name  => 'editor_cursor_blink',
+	type  => Padre::Constant::INTEGER,
+	store => Padre::Constant::HUMAN,
+	default => 500, # milliseconds - this is the actual default for the wxStyledTextCtrl - set to 0 to turn off
+);
 setting(
 	name    => 'find_case',
 	type    => Padre::Constant::BOOLEAN,
@@ -547,7 +822,7 @@ setting(
 	default => 1,
 );
 setting(
-	name    => 'find_quick',
+	name    => 'find_nomatch',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 0,
@@ -564,6 +839,8 @@ setting(
 	store   => Padre::Constant::HUMAN,
 	default => 2,
 );
+
+# Autocomplete settings (global and Perl-specific)
 setting(
 	name    => 'autocomplete_multiclosebracket',
 	type    => Padre::Constant::BOOLEAN,
@@ -582,6 +859,36 @@ setting(
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
+setting(
+	name    => 'autocomplete_subroutine',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
+setting(
+	name    => 'perl_autocomplete_max_suggestions',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => 20,
+);
+setting(
+	name    => 'perl_autocomplete_min_chars',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => 1,
+);
+setting(
+	name    => 'perl_autocomplete_min_suggestion_len',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => 3,
+);
+setting(
+	name    => 'perl_ppi_lexer_limit',
+	type    => Padre::Constant::POSINT,
+	store   => Padre::Constant::HUMAN,
+	default => 4000,
+);
 
 # Behaviour Tuning
 # When running a script from the application some of the files might have
@@ -609,6 +916,23 @@ setting(
 	default => '',
 );
 
+setting(
+	name  => 'perl_tags_file',
+	type  => Padre::Constant::ASCII,
+	store => Padre::Constant::HOST,
+
+	# Don't save a default to allow future updates
+	default => '',
+);
+
+setting(
+	name    => 'xs_calltips_perlapi_version',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::PROJECT,
+	default => 'newest',
+	project => 1,
+);
+
 setting(
 	name    => 'info_on_statusbar',
 	type    => Padre::Constant::BOOLEAN,
@@ -630,8 +954,28 @@ setting(
 	default => 0,
 );
 
+setting(
+	name    => 'mid_button_paste',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
+setting(
+	name    => 'todo_regexp',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => "#\\s*(?:TO[- ]?DO|XXX|FIX[- ]?ME)(?:[ \\t]*[:-]?)(?:[ \\t]*)(.*?)\\s*\$",
+);
+setting(
+	name    => 'sessionmanager_sortorder',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => "0,0",
+);
+
+
 # By default use background threads unless profiling
-# TODO - Make the default actually change
+# TO DO - Make the default actually change
 
 # (Ticket # 669)
 setting(
@@ -639,6 +983,7 @@ setting(
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 1,
+	startup => 1,
 );
 setting(
 	name    => 'locale',
@@ -673,39 +1018,25 @@ setting(
 	name    => 'main_top',
 	type    => Padre::Constant::INTEGER,
 	store   => Padre::Constant::HOST,
-	default => 40,
+	default => -1,
 );
 setting(
 	name    => 'main_left',
 	type    => Padre::Constant::INTEGER,
 	store   => Padre::Constant::HOST,
-	default => 20,
+	default => -1,
 );
 setting(
 	name    => 'main_width',
 	type    => Padre::Constant::POSINT,
 	store   => Padre::Constant::HOST,
-	default => 600,
+	default => -1,
 );
 setting(
 	name    => 'main_height',
 	type    => Padre::Constant::POSINT,
 	store   => Padre::Constant::HOST,
-	default => 400,
-);
-
-# Logging
-setting(
-	name    => 'logging',
-	type    => Padre::Constant::BOOLEAN,
-	store   => Padre::Constant::HOST,
-	default => 0,
-);
-setting(
-	name    => 'logging_trace',
-	type    => Padre::Constant::BOOLEAN,
-	store   => Padre::Constant::HOST,
-	default => 0,
+	default => -1,
 );
 
 # Run Parameters
@@ -725,7 +1056,7 @@ setting(
 	name    => 'run_use_external_window',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HOST,
-	default => 0,
+	default => 1,
 );
 setting(
 	name    => 'external_diff_tool',
@@ -734,27 +1065,53 @@ setting(
 	default => '',
 );
 
-# Enable/Disable functions
+# Enable/Disable entire functions that some people dislike.
+# Normally these should be enabled by default (or should be
+# planned to eventually be enabled by default).
 setting(
-	name    => 'func_config',
+	name    => 'feature_config',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 0,
 );
 setting(
-	name    => 'func_bookmark',
+	name    => 'feature_bookmark',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 1,
 );
 setting(
-	name    => 'func_fontsize',
+	name    => 'feature_fontsize',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 1,
 );
 setting(
-	name    => 'func_session',
+	name    => 'feature_session',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 1,
+);
+setting(
+	name    => 'feature_cursormemory',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 1,
+);
+setting(
+	name    => 'feature_wizard_selector',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
+setting(
+	name    => 'feature_quick_fix',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
+setting(
+	name    => 'feature_restart_hung_task_manager',
 	type    => Padre::Constant::BOOLEAN,
 	store   => Padre::Constant::HUMAN,
 	default => 1,
@@ -885,166 +1242,70 @@ setting(
 	default => 1,
 );
 
-#####################################################################
-# Constructor and Accessors
+# Non-preference settings
+setting(
+	name    => 'session_autosave',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
 
-sub new {
-	my $class = shift;
-	my $host  = shift;
-	my $human = shift;
-	unless ( Params::Util::_INSTANCE( $host, 'Padre::Config::Host' ) ) {
-		Carp::croak("Did not provide a host config to Padre::Config->new");
-	}
-	unless ( Params::Util::_INSTANCE( $human, 'Padre::Config::Human' ) ) {
-		Carp::croak("Did not provide a user config to Padre::Config->new");
-	}
+# The "config_" namespace is for the locations of configuration content
+# outside of the configuration API, and the paths to other non-Padre config
+# files for various external tools (usually so that projects can define the
+# the location of their project-specific policies).
 
-	# Create the basic object with the two required elements
-	my $self = bless [ $host, $human, undef ], $class;
+# The location of the server that will share config data between installs
+setting(
+	name    => 'config_sync_server',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => 'http://escher.ath.cx:3000',
+);
 
-	# Add the optional third element
-	if (@_) {
-		my $project = shift;
-		unless ( Params::Util::_INSTANCE( $project, 'Padre::Config::Project' ) ) {
-			Carp::croak("Did not provide a project config to Padre::Config->new");
-		}
-		$self->[Padre::Constant::PROJECT] = $project;
-	}
+setting(
+	name    => 'config_sync_username',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => '',
+);
 
-	return $self;
-}
+setting(
+	name    => 'config_sync_password',
+	type    => Padre::Constant::ASCII,
+	store   => Padre::Constant::HUMAN,
+	default => '',
+);
 
-sub read {
-	my $class = shift;
+# Location of the Perl::Tidy RC file, if a project wants to set a custom one.
+# When set to false, allow Perl::Tidy to use its own default config location.
+# Load this from the project backend in preference to the host one, so that
+# projects can set their own project-specific config file.
+setting(
+	name    => 'config_perltidy',
+	type    => Padre::Constant::PATH,
+	store   => Padre::Constant::PROJECT,
+	default => '',
+);
 
-	unless ($SINGLETON) {
+# Location of the Perl::Critic RC file, if a project wants to set a custom
+# one. When set to false, allow Perl::Critic to use its own default config
+# location. Load this from the project backend in preference to the host one,
+# so that projects can set their own project-specific config file.
+setting(
+	name    => 'config_perlcritic',
+	type    => Padre::Constant::PATH,
+	store   => Padre::Constant::PROJECT,
+	default => '',
+);
 
-		# Load the host configuration
-		my $host = Padre::Config::Host->read;
-
-		# Load the user configuration
-		my $human = Padre::Config::Human->read
-			|| Padre::Config::Human->create;
-
-		# Hand off to the constructor
-		$SINGLETON = $class->new( $host, $human );
-
-		$SINGLETON->Padre::Config::Upgrade::check();
-	}
-
-	return $SINGLETON;
-}
-
-sub write {
-	my $self = shift;
-
-	# Save the user configuration
-	$self->[Padre::Constant::HUMAN]->{version} = $REVISION;
-	$self->[Padre::Constant::HUMAN]->write;
-
-	# Save the host configuration
-	$self->[Padre::Constant::HOST]->{version} = $REVISION;
-	$self->[Padre::Constant::HOST]->write;
-
-	return 1;
-}
-
-# Fetches an explicitly named default
-sub default {
-	my $self = shift;
-	my $name = shift;
-
-	# Does the setting exist?
-	unless ( $SETTING{$name} ) {
-		Carp::croak("The configuration setting '$name' does not exist");
-	}
-
-	return $DEFAULT{$name};
-}
-
-
-
-
-
-######################################################################
-# Main Methods
-
-sub set {
-	my $self  = shift;
-	my $name  = shift;
-	my $value = shift;
-
-	# Does the setting exist?
-	my $setting = $SETTING{$name};
-	unless ($setting) {
-		Carp::croak("The configuration setting '$name' does not exist");
-	}
-
-	# All types are Padre::Constant::ASCII-like
-	unless ( defined $value and not ref $value ) {
-		Carp::croak("Missing or non-scalar value for setting '$name'");
-	}
-
-	# We don't need to do additional checks on Padre::Constant::ASCII
-	my $type = $setting->type;
-	if ( $type == Padre::Constant::BOOLEAN ) {
-		$value = 0 if $value eq '';
-		if ( $value ne '1' and $value ne '0' ) {
-			Carp::croak("Tried to change setting '$name' to non-boolean '$value'");
-		}
-	}
-	if ( $type == Padre::Constant::POSINT and not Params::Util::_POSINT($value) ) {
-		Carp::croak("Tried to change setting '$name' to non-posint '$value'");
-	}
-	if ( $type == Padre::Constant::INTEGER and not _INTEGER($value) ) {
-		Carp::croak("Tried to change setting '$name' to non-integer '$value'");
-	}
-	if ( $type == Padre::Constant::PATH and not -e $value ) {
-		Carp::croak("Tried to change setting '$name' to non-existant path '$value'");
-	}
-
-	# Set the value into the appropriate backend
-	my $store = $SETTING{$name}->store;
-	$self->[$store]->{$name} = $value;
-
-	return 1;
-}
-
-# Set a value in the configuration and apply the preference change
-# to the application.
-sub apply {
-	my $self    = shift;
-	my $name    = shift;
-	my $value   = shift;
-	my $current = _CURRENT(@_);
-
-	# Set the config value
-	$self->set( $name => $value );
-
-	# Does this setting have an apply hook
-	my $code = $SETTING{$name}->apply;
-	if ($code) {
-		$code->( $current->main, $value );
-	}
-
-	return 1;
-}
-
-
-
-
-
-######################################################################
-# Support Functions
-
-#
-# my $is_integer = _INTEGER( $scalar );
-#
-# return true if $scalar is an integer.
-#
-sub _INTEGER ($) {
-	return defined $_[0] && !ref $_[0] && $_[0] =~ m/^(?:0|-?[1-9]\d*)$/;
-}
+# Save if feedback has been send or not
+setting(
+	name    => 'feedback_done',
+	type    => Padre::Constant::BOOLEAN,
+	store   => Padre::Constant::HUMAN,
+	default => 0,
+);
 
 1;
 
@@ -1069,22 +1330,25 @@ the functions for loading and saving the configuration.
 
 The Padre configuration lives in two places:
 
- - A user-editable text file usually called config.yml
+=over
 
- - A SQlite - database which shouldn't be edited by the
-   user.
+=item a user-editable text file usually called F<config.yml>
+
+=item an SQLite database which shouldn't be edited by the user
+
+=back
 
 =head2 Generic usage
 
-Every setting is accessed by a method named after it, which is a mutator.
-ie, it can be used both as a getter and a setter, depending on the number
+Every setting is accessed by a mutator named after it,
+i.e. it can be used both as a getter and a setter depending on the number
 of arguments passed to it.
 
 =head2 Different types of settings
 
 Padre needs to store different settings. Those preferences are stored in
 different places depending on their impact. But C<Padre::Config> allows to
-access them with a unified api (a mutator). Only their declaration differ
+access them with a unified API (a mutator). Only their declaration differs
 in the module.
 
 Here are the various types of settings that C<Padre::Config> can manage:
@@ -1094,7 +1358,7 @@ Here are the various types of settings that C<Padre::Config> can manage:
 =item * User settings
 
 Those settings are general settings that relates to user preferences. They range
-from general user interface look&feel (whether to show the line numbers, etc.)
+from general user interface I<look & feel> (whether to show the line numbers, etc.)
 to editor preferences (tab width, etc.) and other personal settings.
 
 Those settings are stored in a YAML file, and accessed with C<Padre::Config::Human>.
@@ -1126,14 +1390,14 @@ a own value.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 
 This program is free software; you can redistribute it and/or modify it under the
 same terms as Perl 5 itself.
 
 =cut
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
