@@ -4,7 +4,7 @@ use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 
 sub select_names {
 	Padre::DB->selectcol_arrayref('select name from bookmark order by name');
@@ -33,18 +33,53 @@ TO BE COMPLETED
 
 TO BE COMPLETED
 
-=head1 INTERFACE
+=head1 METHODS
 
-=head2 Methods
+=head2 base
 
-=head3 select
+  # Returns 'Padre::DB'
+  my $namespace = Padre::DB::Bookmark->base;
+
+Normally you will only need to work directly with a table class,
+and only with one ORLite package.
+
+However, if for some reason you need to work with multiple ORLite packages
+at the same time without hardcoding the root namespace all the time, you
+can determine the root namespace from an object or table class with the
+C<base> method.
+
+=head2 table
+
+  # Returns 'bookmark'
+  print Padre::DB::Bookmark->table;
+
+While you should not need the name of table for any simple operations,
+from time to time you may need it programatically. If you do need it,
+you can use the C<table> method to get the table name.
+
+=head2 load
+
+  my $object = Padre::DB::Bookmark->load( $id );
+
+If your table has single column primary key, a C<load> method will be
+generated in the class. If there is no primary key, the method is not
+created.
+
+The C<load> method provides a shortcut mechanism for fetching a single
+object based on the value of the primary key. However it should only
+be used for cases where your code trusts the record to already exists.
+
+It returns a C<Padre::DB::Bookmark> object, or throws an exception if the
+object does not exist.
+
+=head2 select
 
   # Get all objects in list context
   my @list = Padre::DB::Bookmark->select;
-
+  
   # Get a subset of objects in scalar context
   my $array_ref = Padre::DB::Bookmark->select(
-      'where name > ? order by name',
+      'where id > ? order by id',
       1000,
   );
 
@@ -57,18 +92,64 @@ to be bound to the placeholders in the SQL phrase. Any SQL that is
 compatible with SQLite can be used in the parameter.
 
 Returns a list of B<Padre::DB::Bookmark> objects when called in list context, or a
-reference to an ARRAY of B<Padre::DB::Bookmark> objects when called in scalar context.
+reference to an C<ARRAY> of B<Padre::DB::Bookmark> objects when called in scalar
+context.
 
 Throws an exception on error, typically directly from the L<DBI> layer.
 
-=head3 count
+=head2 iterate
+
+  Padre::DB::Bookmark->iterate( sub {
+      print $_->id . "\n";
+  } );
+
+The C<iterate> method enables the processing of large tables one record at
+a time without loading having to them all into memory in advance.
+
+This plays well to the strength of SQLite, allowing it to do the work of
+loading arbitrarily large stream of records from disk while retaining the
+full power of Perl when processing the records.
+
+The last argument to C<iterate> must be a subroutine reference that will be
+called for each element in the list, with the object provided in the topic
+variable C<$_>.
+
+This makes the C<iterate> code fragment above functionally equivalent to the
+following, except with an O(1) memory cost instead of O(n).
+
+  foreach ( Padre::DB::Bookmark->select ) {
+      print $_->id . "\n";
+  }
+
+You can filter the list via SQL in the same way you can with C<select>.
+
+  Padre::DB::Bookmark->iterate(
+      'order by ?', 'id',
+      sub {
+          print $_->id . "\n";
+      }
+  );
+
+You can also use it in raw form from the root namespace for better control.
+Using this form also allows for the use of arbitrarily complex queries,
+including joins. Instead of being objects, rows are provided as C<ARRAY>
+references when used in this form.
+
+  Padre::DB->iterate(
+      'select name from bookmark order by id',
+      sub {
+          print $_->[0] . "\n";
+      }
+  );
+
+=head2 count
 
   # How many objects are in the table
   my $rows = Padre::DB::Bookmark->count;
-
-  # How many objects
+  
+  # How many objects 
   my $small = Padre::DB::Bookmark->count(
-      'where name > ?',
+      'where id > ?',
       1000,
   );
 
@@ -84,19 +165,21 @@ Returns the number of objects that match the condition.
 
 Throws an exception on error, typically directly from the L<DBI> layer.
 
-=head3 new
+=head2 new
 
-TO BE COMPLETED
+  TO BE COMPLETED
 
 The C<new> constructor is used to create a new abstract object that
 is not (yet) written to the database.
 
 Returns a new L<Padre::DB::Bookmark> object.
 
-=head3 create
+=head2 create
 
   my $object = Padre::DB::Bookmark->create(
 
+      id => 'value',
+
       name => 'value',
 
       file => 'value',
@@ -107,37 +190,37 @@ Returns a new L<Padre::DB::Bookmark> object.
 
 The C<create> constructor is a one-step combination of C<new> and
 C<insert> that takes the column parameters, creates a new
-L<Padre::DB::Bookmark> object, inserts the appropriate row into the L<bookmark>
-table, and then returns the object.
+L<Padre::DB::Bookmark> object, inserts the appropriate row into the
+L<bookmark> table, and then returns the object.
 
-If the primary key column C<name> is not provided to the
+If the primary key column C<id> is not provided to the
 constructor (or it is false) the object returned will have
-C<name> set to the new unique identifier.
+C<id> set to the new unique identifier.
+ 
+Returns a new L<bookmark> object, or throws an exception on
+error, typically from the L<DBI> layer.
 
-Returns a new L<bookmark> object, or throws an exception on error,
-typically from the L<DBI> layer.
-
-=head3 insert
+=head2 insert
 
   $object->insert;
 
 The C<insert> method commits a new object (created with the C<new> method)
 into the database.
 
-If a the primary key column C<name> is not provided to the
+If a the primary key column C<id> is not provided to the
 constructor (or it is false) the object returned will have
-C<name> set to the new unique identifier.
+C<id> set to the new unique identifier.
 
 Returns the object itself as a convenience, or throws an exception
 on error, typically from the L<DBI> layer.
 
-=head3 delete
+=head2 delete
 
   # Delete a single instantiated object
   $object->delete;
-
+  
   # Delete multiple rows from the bookmark table
-  Padre::DB::Bookmark->delete('where name > ?', 1000);
+  Padre::DB::Bookmark->delete('where id > ?', 1000);
 
 The C<delete> method can be used in a class form and an instance form.
 
@@ -154,7 +237,7 @@ in the parameter.
 Returns true on success or throws an exception on error, or if you
 attempt to call delete without a SQL condition phrase.
 
-=head3 truncate
+=head2 truncate
 
   # Delete all records in the bookmark table
   Padre::DB::Bookmark->truncate;
@@ -169,11 +252,11 @@ all records in a table with specific intent.
 
 Returns true, or throws an exception on error.
 
-=head2 Accessors
+=head1 ACCESSORS
 
-=head3 name
+=head2 id
 
-  if ( $object->name ) {
+  if ( $object->id ) {
       print "Object has been inserted\n";
   } else {
       print "Object has not been inserted\n";
@@ -181,18 +264,18 @@ Returns true, or throws an exception on error.
 
 Returns true, or throws an exception on error.
 
+REMAINING ACCESSORS TO BE COMPLETED
 
-  REMAINING ACCESSORS TO BE COMPLETED
-
-=head2 SQL
+=head1 SQL
 
 The bookmark table was originally created with the
 following SQL command.
 
   CREATE TABLE bookmark (
-  	name VARCHAR(255) NOT NULL PRIMARY KEY,
-  	file VARCHAR(255) NOT NULL,
-  	line INTEGER NOT NULL
+      id INTEGER NOT NULL PRIMARY KEY,
+      name VARCHAR(255) UNIQUE NOT NULL,
+      file VARCHAR(255) NOT NULL,
+      line INTEGER NOT NULL
   )
 
 =head1 SUPPORT
@@ -203,7 +286,7 @@ See the documentation for L<Padre::DB> for more information.
 
 =head1 AUTHOR
 
-Adam Kennedy
+Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
