@@ -45,11 +45,12 @@ use Padre::Constant ();
 use Padre::Util     ('_T');
 use Padre::Config   ();
 use Padre::Wx       ();
+use Padre::Logger;
 
 use constant DEFAULT  => 'en-gb';
-use constant SHAREDIR => File::Spec->rel2abs( Padre::Util::sharedir('locale') );
+use constant SHAREDIR => Padre::Util::sharedir('locale');
 
-our $VERSION = '0.50';
+our $VERSION = '0.76';
 
 # The RFC4646 table is the primary language data table and contains
 # mappings from a Padre-supported language to all the relevant data
@@ -58,6 +59,17 @@ our $VERSION = '0.50';
 # simplicity (for now) we list them all as lower-case.
 my %RFC4646;
 
+# The utf8text could/should be taken from gettext translations of the iso-codes package
+# file:///usr/share/locale/*/LC_MESSAGES/iso_639.mo
+# file:///usr/share/xml/iso-codes/iso_639.xml
+# http://pkg-isocodes.alioth.debian.org/
+
+sub label {
+	my $name = shift;
+	require Encode;
+	return $RFC4646{$name}{utf8text} ? Encode::decode( 'utf8', $RFC4646{$name}{utf8text} ) : $name;
+}
+
 BEGIN {
 	%RFC4646 = (
 
@@ -109,8 +121,8 @@ BEGIN {
 		# Example entry for an language which is not supported directly,
 		# but which Padre is aware of.
 		'en-au' => {
-			gettext  => _T('English (Australian)'),
-			utf8text => 'English (Australian)',
+			gettext  => _T('English (Australia)'),
+			utf8text => 'English (Australia)',
 			iso639   => 'en',
 			iso3166  => 'AU',
 			wxid     => Wx::wxLANGUAGE_ENGLISH_AUSTRALIA,
@@ -160,6 +172,16 @@ BEGIN {
 			supported => 1,
 		},
 
+		'da' => {
+			gettext   => _T('Danish'),
+			utf8text  => 'Dansk',
+			iso639    => 'da',
+			iso3166   => 'DA',
+			wxid      => Wx::wxLANGUAGE_DANISH,
+			fallback  => [ 'en-gb', 'en-us' ],
+			supported => 0,
+		},
+
 		'de' => {
 			gettext   => _T('German'),
 			utf8text  => 'Deutsch',
@@ -243,7 +265,7 @@ BEGIN {
 		},
 
 		'fr-ca' => {
-			gettext   => _T('French (France)'),
+			gettext   => _T('French (Canada)'),
 			utf8text  => 'Français (Canada)',
 			iso639    => 'fr',
 			iso3166   => 'CA',
@@ -322,7 +344,7 @@ BEGIN {
 
 		'nl-nl' => {
 
-			# Simplify until there's another Italian
+			# Simplify until there's another Dutch
 			# gettext   => 'Dutch (Netherlands)',
 			# utf8text  => 'Nederlands (Nederlands)',
 			gettext   => _T('Dutch'),
@@ -337,16 +359,16 @@ BEGIN {
 		'nl-be' => {
 			gettext   => _T('Dutch (Belgium)'),
 			utf8text  => 'Nederlands (België)',
-			iso639    => 'en',
+			iso639    => 'nl',
 			iso3166   => 'BE',
 			wxid      => Wx::wxLANGUAGE_DUTCH_BELGIAN,
 			fallback  => ['nl-nl'],
-			supported => 1,
+			supported => 0,
 		},
 
 		'no' => {
-			gettext   => _T('Norwegian (Norway)'),
-			utf8text  => 'Norsk (Norge)',
+			gettext   => _T('Norwegian'),
+			utf8text  => 'Norsk',
 			iso639    => 'no',
 			iso3166   => 'NO',
 			wxid      => Wx::wxLANGUAGE_NORWEGIAN_BOKMAL,
@@ -394,6 +416,16 @@ BEGIN {
 			supported => 1,
 		},
 
+		'tr' => {
+			gettext   => _T('Turkish'),
+			utf8text  => 'Türkçe',
+			iso639    => 'tr',
+			iso3166   => 'TR',
+			wxid      => Wx::wxLANGUAGE_TURKISH,
+			fallback  => [],
+			supported => 1,
+		},
+
 		'zh' => {
 			gettext   => _T('Chinese'),
 			utf8text  => 'Chinese',
@@ -430,7 +462,7 @@ BEGIN {
 		# a language is not supported by various older standards.
 		'x-klingon' => {
 			gettext   => _T('Klingon'),
-			utf8text  => 'Klingon',    # TODO Fix this at some point
+			utf8text  => 'Klingon',    # TO DO Fix this at some point
 			iso639    => undef,
 			iso3166   => undef,
 			wxid      => undef,
@@ -439,8 +471,8 @@ BEGIN {
 		},
 	);
 
-	# Post-process to find the language each language
-	# will actually fall back to.
+	# Post-process to find the language that each language
+	# will actually fall back to, rather than prefer to fall back to.
 	foreach my $id ( keys %RFC4646 ) {
 		my $lang = $RFC4646{$id};
 		$lang->{actual} = List::Util::first {
@@ -453,9 +485,12 @@ BEGIN {
 use constant WX => Wx::Locale::GetSystemLanguage();
 
 use constant system_rfc4646 => List::Util::first {
-	$RFC4646{$_}->{wxid} == WX;
+	defined $RFC4646{$_}->{wxid}
+		&& $RFC4646{$_}->{wxid} == WX;
 }
-grep { defined $RFC4646{$_}->{wxid} } sort keys %RFC4646;
+sort keys %RFC4646;
+
+use constant last_resort_rfc4646 => 'en-gb';
 
 #####################################################################
 # Locale 2.0 Implementation
@@ -463,7 +498,7 @@ grep { defined $RFC4646{$_}->{wxid} } sort keys %RFC4646;
 # Find the rfc4646 to use by default
 sub rfc4646 {
 	my $config = Padre::Config->read;
-	my $locale = $config->locale;
+	my $locale = $_[0] || $config->locale;
 
 	if ( $locale and not $RFC4646{$locale} ) {
 
@@ -495,7 +530,9 @@ sub system_iso639 {
 # Given a rfc4646 identifier, sets the language globally
 # and returns the relevant Wx::Locale object.
 sub object {
-	my $id     = rfc4646();
+	my $langcode = shift;
+	undef $langcode if ref($langcode);
+	my $id     = rfc4646($langcode);
 	my $lang   = $RFC4646{$id}->{wxid};
 	my $locale = Wx::Locale->new($lang);
 	$locale->AddCatalogLookupPathPrefix( Padre::Util::sharedir('locale') );
@@ -564,7 +601,7 @@ sub encoding_system_default {
 		return;
 	}
 
-	Padre::Util::debug("Encoding system default: ($encoding)");
+	TRACE("Encoding system default: ($encoding)") if DEBUG;
 
 	return $encoding;
 }
@@ -572,7 +609,11 @@ sub encoding_system_default {
 sub encoding_from_string {
 	my $content = shift;
 
-	# FIXME
+	# Because Encode::Guess is slow and expensive, do an initial fast
+	# regexp scan for the simplest and most common "ascii" encoding.
+	return 'ascii' unless $content =~ /[^[:ascii:]]/;
+
+	# FIX ME
 	# This is a just heuristic approach. Maybe there is a better way. :)
 	# Japanese and Chinese have to be tested. Only Korean is tested.
 	#
@@ -599,10 +640,10 @@ sub encoding_from_string {
 	push @guesses, 'latin1';
 	my $guess = Encode::Guess::guess_encoding( $content, @guesses );
 	unless ( defined $guess ) {
-		$guess = '';                 # to avoid warnings
+		$guess = '';                 # To avoid warnings
 	}
 
-	Padre::Util::debug("Encoding guess: ($guess)");
+	TRACE("Encoding guess: ($guess)") if DEBUG;
 
 	# Wow, nice!
 	if ( ref($guess) and ref($guess) =~ m/^Encode::/ ) {
@@ -631,14 +672,14 @@ sub encoding_from_string {
 		$encoding = 'utf-8';
 	}
 
-	Padre::Util::debug("Encoding selected: ($encoding)");
+	TRACE("Encoding selected: ($encoding)") if DEBUG;
 
 	return $encoding;
 }
 
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
