@@ -53,7 +53,7 @@ use YAML::Tiny     ();
 use Padre::DB      ();
 use Padre::Wx      ();
 
-our $VERSION    = '0.63';
+our $VERSION    = '0.76';
 our $COMPATIBLE = '0.43';
 
 # Link plug-ins back to their IDE
@@ -106,8 +106,12 @@ sub plugin_directory_share {
 	$class =~ s/\=HASH\(.+?\)$//;
 
 	if ( $ENV{PADRE_DEV} ) {
+		my $bin = do {
+			no warnings;
+			$FindBin::Bin;
+		};
 		my $root = File::Spec->catdir(
-			$FindBin::Bin,
+			$bin,
 			File::Spec->updir,
 			File::Spec->updir,
 			$class,
@@ -138,7 +142,9 @@ know where to look for your plug-in l10n catalog.
 
 It defaults to F<$sharedir/locale> (with C<$sharedir> as defined by
 C<File::ShareDir> and thus should work as is for your plug-in if you're
-using the C<install_share> command of L<Module::Install>.
+using the C<install_share> command of L<Module::Install>. If you are
+using L<Module::Build> version 0.36 and later, please use the C<share_dir>
+new() argument.
 
 Your plug-in catalogs should be named F<$plugin-$locale.po> (or F<.mo>
 for the compiled form) where C<$plugin> is the class name of your plug-in with
@@ -180,6 +186,20 @@ sub plugin_icon {
 
 =pod
 
+=head2 C<plugin_wizards>
+
+The C<plugin_wizards> method will be called by Padre when it retrieves the wizard list.
+
+The default implementation returns an empty list
+
+=cut
+
+sub plugin_wizards {
+	return ();
+}
+
+=pod
+
 =head2 C<padre_interfaces>
 
   sub padre_interfaces {
@@ -211,8 +231,20 @@ disabled unless the user has specifically allowed experimental plug-ins.
 
 =cut
 
-sub padre_interfaces {
-	return ();
+# Disabled so that we can detect plugins created before the existance
+# of the compatibility mechanism.
+# sub padre_interfaces {
+#     return ();
+# }
+
+# Convenience integration with Class::Unload
+sub unload {
+	require Class::Unload;
+	my $either = shift;
+	foreach my $package (@_) {
+		Class::Unload->unload($package);
+	}
+	return 1;
 }
 
 
@@ -687,7 +719,11 @@ sub _menu_plugins_submenu {
 
 			# Convert to a function reference
 			my $method = $value;
-			$value = sub { $self->$method(@_) };
+			$value = sub {
+				local $@;
+				eval { $self->$method(@_); };
+				$main->error("Unhandled exception in plugin menu: $@") if $@;
+			};
 		}
 
 		# Function Reference
@@ -696,8 +732,9 @@ sub _menu_plugins_submenu {
 				$main,
 				$menu->Append( -1, $label ),
 				sub {
-					eval { $value->(@_) };
-					Carp::cluck($@) if $@;
+					local $@;
+					eval { $value->(@_); };
+					$main->error("Unhandled exception in plugin menu: $@") if $@;
 				},
 			);
 			next;
@@ -821,7 +858,14 @@ L<Padre::Wx::Main> (main window) object.
 =cut
 
 sub main {
-	$IDE{ Scalar::Util::refaddr( $_[0] ) }->wx->main;
+	my $self = shift;
+
+	# TODO sometimes Padre crashes here claiming that thing is undef:
+	if ( not defined $IDE{ Scalar::Util::refaddr($self) } ) {
+		Carp::cluck("UNDEF !!! $_[0]");
+		return $self->main; # fixes warning from badcode tests
+	}
+	$IDE{ Scalar::Util::refaddr($self) }->wx->main;
 }
 
 =pod
@@ -850,7 +894,7 @@ L<Padre>
 Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 
 This program is free software; you can redistribute
-it and/or modify it under the same terms as Perl itself.
+it and/or modify it under the same terms as Perl 5 itself.
 
 The full text of the license can be found in the
 LICENSE file included with this module.
