@@ -113,13 +113,12 @@ this plug-in entirely.
 use 5.008;
 use strict;
 use warnings;
-use Config        ();
-use Scalar::Util  ();
-use Padre::Plugin ();
-use Padre::Task::HTTPClient;
-use Padre::Constant();
+use Config          ();
+use Scalar::Util    ();
+use Padre::Plugin   ();
+use Padre::Constant ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 our @ISA     = 'Padre::Plugin';
 
 # Track the number of times actions are used
@@ -132,12 +131,18 @@ our %ACTION = ();
 ######################################################################
 # Padre::Plugin Methods
 
-sub plugin_name {
-	'Padre Popularity Contest';
+sub padre_interfaces {
+	return (
+		'Padre::Plugin'           => 0.66,
+		'Padre::Task'             => 0.66,
+		'Padre::Task::LWP'        => 0.66,
+		'Padre::Util::SVN'        => 0.66,
+		'Padre::Wx::Dialog::Text' => 0.66,
+	);
 }
 
-sub plugin_interfaces {
-	'Padre::Plugin' => 0.43;
+sub plugin_name {
+	'Padre Popularity Contest';
 }
 
 sub plugin_enable {
@@ -182,8 +187,7 @@ sub plugin_disable {
 	}
 
 	# Make sure our task class is unloaded
-	require Class::Unload;
-	Class::Unload->unload('Padre::Plugin::PopularityContext::Ping');
+	$self->unload('Padre::Plugin::PopularityContext::Ping');
 
 	$self->SUPER::plugin_disable;
 }
@@ -276,10 +280,12 @@ sub report {
 	my $report = $self->_generate;
 
 	# TO DO: Enable as soon as the server is functional:
-	#	my $response = Padre::Task::HTTPClient->new(
-	#		URL   => 'http://padre.perlide.org/popularity_contest.cgi',
-	#		query => \%STATS, method => 'POST'
-	#	)->run;
+	#	$self->task_request(
+	#		task   => 'Padre::Task::LWP'->new(
+	#		method => 'POST',
+	#		url    => 'http://padre.perlide.org/popularity_contest.cgi',
+	#		query  => \%STATS,
+	#	);
 
 	return 1;
 }
