@@ -9,19 +9,23 @@ use 5.008;
 use strict;
 use warnings;
 use Padre::Task ();
-use Padre::Wx   ();
 
-our $VERSION = '0.50';
+our $VERSION = '0.76';
 our @ISA     = 'Padre::Task';
 
 sub run {
+
+	# We don't need to load all of Padre::Wx for this,
+	# but we do need the minimum bits of wxWidgets.
+	require Wx;
+
 	Wx::LaunchDefaultBrowser( $_[0]->{url} );
 	return 1;
 }
 
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
