@@ -21,9 +21,9 @@ I'll document this more later...
 use 5.008;
 use strict;
 use warnings;
-use Params::Util qw{ _INSTANCE };
+use Params::Util ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 
 #####################################################################
 # Constructor
@@ -60,9 +60,9 @@ or throws an exception on error.
 
 sub apply {
 	my $self = shift;
-	my $document = _INSTANCE( shift, 'Padre::Document' );
+	my $document = Params::Util::_INSTANCE( shift, 'Padre::Document' );
 	unless ($document) {
-		die('Did not provide a Padre::Document object to apply');
+		die 'Did not provide a Padre::Document object to apply';
 	}
 
 	# Null transform
