@@ -13,7 +13,7 @@ use Padre::Util             ();
 use Wx::Perl::ProcessStream ();
 use PPI                     ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 our @ISA     = 'Wx::Dialog';
 
 sub new {
@@ -143,10 +143,14 @@ sub _content_developers {
           <p>
             <br>
             <br>
+            Aaron Trevena<br>
+            <br>
             Adam Kennedy<br>
             <br>
             Ahmad Zawawi - أحمد محمد زواوي<br>
             <br>
+            Alexandr Ciornii<br>
+            <br>
             Blake Willmarth<br>
             <br>
             Breno G. de Oliveira<br>
@@ -158,14 +162,14 @@ sub _content_developers {
             Chris Dolan<br>
             <br>
             Claudio Ramirez<br>
-            <br>
-            code4pay (pseudonym only)<br>            
           </p>
         </td>
         <td valign="top">
           <p>
             <br>
             <br>
+            code4pay (pseudonym only)<br>
+            <br>
             Fayland Lam<br>
             <br>
             Gabriel Vieira<br>
@@ -201,6 +205,8 @@ sub _content_developers {
             <br>
             Peter Lavender<br>
             <br>
+            Ryan Niebur<br>
+            <br>
             Sebastian Willing<br>
             <br>
             Steffen Müller<br>
@@ -309,12 +315,16 @@ END_HTML
 sub _content_info {
 	my $self           = shift;
 	my $padre_info     = Wx::gettext('System Info');
-	my $wx_widgets     = Wx::wxVERSION_STRING();
-	my $config_dir_txt = Wx::gettext('Config dir:');
+	my $config_dir_txt = Wx::gettext('Config:');
 	my $config_dir     = Padre::Constant::CONFIG_DIR;
-	my $uptime         = time - $^T;
-	my @uptime_parts   = ( 0, 0, 0 );
 
+	# Reformat the native wxWidgets version string slightly
+	my $wx_widgets = Wx::wxVERSION_STRING();
+	$wx_widgets =~ s/^wx\w+\s+//;
+
+	# Calculate the process uptime
+	my $uptime = time - $^T;
+	my @uptime_parts = ( 0, 0, 0 );
 	if ( $uptime > 3600 ) {
 		$uptime_parts[0] = int( $uptime / 3600 );
 		$uptime -= $uptime_parts[0] * 3600;
@@ -327,12 +337,17 @@ sub _content_info {
 	my $uptime_text = Wx::gettext('Uptime');
 	$uptime = sprintf( '%d:%02d:%02d', @uptime_parts );
 
+	# Calculate the current memory in use across all threads
 	my $ram = Padre::Util::humanbytes( Padre::Util::process_memory() ) || '0';
-
 	$ram = '(' . Wx::gettext('unsupported') . ')' if $ram eq '0';
 
 	# Yes, THIS variable should have this upper case char :-)
 	my $Perl_version = $^V || $];
+	$Perl_version = "$Perl_version";
+	$Perl_version =~ s/^v//;
+
+	# How many threads are running
+	my $threads = $INC{'threads.pm'} ? scalar( threads->list ) : 'disabled';
 
 	$self->{info}->SetPage( $self->_rtl(<<"END_HTML") );
 <html>
@@ -394,6 +409,14 @@ sub _content_info {
         $ram
         </td>
       </tr>
+      <tr>
+        <td valign="top">
+        Threads:
+        </td>
+        <td>
+        $threads
+        </td>
+      </tr>
     </table>
   </body>
 </html>
@@ -409,7 +432,7 @@ sub ShowModal {
 
 # Arabic and Hebrew names are not showing up correctly
 sub _rtl {
-	my ( $self, $text ) = @_;
+	my $text = $_[1];
 	$text =~ s/(\p{InArabic}+)\s+(\p{InArabic}+)\s+(\p{InArabic}+)/$3 $2 $1/g;
 	$text =~ s/(\p{InHebrew}+)\s+(\p{InHebrew}+)/$2 $1/g;
 	return $text;
