@@ -4,18 +4,19 @@ use 5.008;
 use strict;
 use warnings;
 use Padre::Current                         ();
+use Padre::Util                            ('_T');
 use Padre::Wx                              ();
 use Padre::Wx::Dialog                      ();
 use Padre::Wx::Editor                      ();
 use Padre::Wx::Dialog::Preferences::Editor ();
 use Padre::MimeTypes                       ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 our @ISA     = 'Padre::Wx::Dialog';
 
 our %PANELS = (
-	'Padre::Wx::Dialog::Preferences::File'             => Wx::gettext('Local/Remote File Access'),
-	'Padre::Wx::Dialog::Preferences::PerlAutoComplete' => Wx::gettext('Perl Auto Complete')
+	'Padre::Wx::Dialog::Preferences::File'             => _T('Local/Remote File Access'),
+	'Padre::Wx::Dialog::Preferences::PerlAutoComplete' => _T('Perl Auto Complete'),
 );
 
 =pod
@@ -347,13 +348,25 @@ sub _behaviour_panel {
 		[   [ 'Wx::StaticText', undef, Wx::gettext('Check for file updates on disk every (seconds):') ],
 			[ 'Wx::SpinCtrl', 'update_file_from_disk_interval', $config->update_file_from_disk_interval, 0, 90 ]
 		],
+		[   [   'Wx::StaticText', undef, Wx::gettext('Cursor Blink Rate (milliseconds - 0 = blink off; 500 = default):')
+			],
+			[ 'Wx::TextCtrl', 'editor_cursor_blink', $config->editor_cursor_blink ]
+		],
 
 		# Will be moved to a own AutoComp-panel as soon as there are enough options for this (and I get the spare time to do it):
+		[   [   'Wx::CheckBox',
+				'autocomplete_brackets',
+				( $config->autocomplete_brackets ? 1 : 0 ),
+				Wx::gettext("Autocomplete brackets")
+			],
+			[]
+		],
 		[   [   'Wx::CheckBox',
 				'autocomplete_multiclosebracket',
 				( $config->autocomplete_multiclosebracket ? 1 : 0 ),
 				Wx::gettext(
-					"Add another closing bracket if there is already one (and the auto-bracket-function is enabled)")
+					"Add another closing bracket if there is already one (and the 'Autocomplete brackets' above is enabled)"
+				)
 			],
 			[]
 		],
@@ -1008,6 +1021,8 @@ sub run {
 	);
 	my @default_line_ending_localized = map { Wx::gettext($_) } @default_line_ending_items;
 
+
+
 	$self->{dialog} = $self->dialog(
 		$win,
 		\@startup_files_localized,
@@ -1079,6 +1094,10 @@ sub run {
 		'editor_beginner',
 		$data->{editor_beginner} ? 1 : 0
 	);
+	$config->set(
+		'editor_cursor_blink',
+		$data->{editor_cursor_blink}
+	);
 	$config->set(
 		'default_projects_directory',
 		$data->{default_projects_directory}
@@ -1174,6 +1193,10 @@ sub run {
 		'update_file_from_disk_interval',
 		$data->{update_file_from_disk_interval}
 	);
+	$config->set(
+		'autocomplete_brackets',
+		$data->{autocomplete_brackets} ? 1 : 0
+	);
 	$config->set(
 		'autocomplete_multiclosebracket',
 		$data->{autocomplete_multiclosebracket} ? 1 : 0
