@@ -5,16 +5,16 @@ package Padre::Wx::Dialog::RegexEditor;
 use 5.008;
 use strict;
 use warnings;
-use Padre::Wx                  ();
-use Padre::Wx::Icon            ();
-use Padre::Wx::Role::MainChild ();
+use Padre::Wx             ();
+use Padre::Wx::Icon       ();
+use Padre::Wx::Role::Main ();
 
 # RichTextCtrl
 use Wx::RichText ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 our @ISA     = qw{
-	Padre::Wx::Role::MainChild
+	Padre::Wx::Role::Main
 	Wx::Dialog
 };
 
@@ -42,6 +42,7 @@ sub new {
 
 	# create sizer that will host all controls
 	my $sizer = Wx::BoxSizer->new(Wx::wxHORIZONTAL);
+	$self->{sizer} = $sizer;
 
 	# Create the controls
 	$self->_create_controls($sizer);
@@ -137,60 +138,15 @@ sub _regex_groups {
 sub _create_controls {
 	my ( $self, $sizer ) = @_;
 
-	# Dialog Controls
+	# Dialog Controls, created in keyboard navigation order
 
 	# Regex text field
 	my $regex_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Regular expression:') );
-	$self->{regex} = Wx::RichTextCtrl->new(
+	$self->{regex} = Wx::TextCtrl->new(
 		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
 		Wx::wxRE_MULTILINE | Wx::wxWANTS_CHARS # Otherwise arrows will not work on win32
 	);
 
-	# Replace regex text field
-	my $replace_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Replace text with:') );
-	$self->{replace} = Wx::TextCtrl->new(
-		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
-		Wx::wxTE_MULTILINE | Wx::wxNO_FULL_REPAINT_ON_RESIZE
-	);
-
-	# Describe-the-regex text field
-	my $description_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Description:') );
-	$self->{description_text} = Wx::TextCtrl->new(
-		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
-		Wx::wxTE_MULTILINE | Wx::wxNO_FULL_REPAINT_ON_RESIZE
-	);
-
-	# Original input text field
-	my $original_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Original text:') );
-	$self->{original_text} = Wx::TextCtrl->new(
-		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
-		Wx::wxTE_MULTILINE | Wx::wxNO_FULL_REPAINT_ON_RESIZE
-	);
-
-	# Matched readonly text field
-	my $matched_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Matched text:') );
-	$self->{matched_text} = Wx::RichTextCtrl->new(
-		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
-		Wx::wxRE_MULTILINE | Wx::wxRE_READONLY | Wx::wxWANTS_CHARS # Otherwise arrows will not work on win32
-	);
-
-	# Result from replace text field
-	my $result_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Result from replace:') );
-	$self->{result_text} = Wx::RichTextCtrl->new(
-		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
-		Wx::wxRE_MULTILINE | Wx::wxRE_READONLY | Wx::wxWANTS_CHARS # Otherwise arrows will not work on win32
-	);
-
-	# Modifiers
-	my %m = $self->_modifiers();
-	foreach my $name ( keys %m ) {
-		$self->{$name} = Wx::CheckBox->new(
-			$self,
-			-1,
-			$m{$name}{name},
-		);
-	}
-
 	my %regex_groups = $self->_regex_groups;
 	foreach my $code ( sort keys %regex_groups ) {
 		my %sub_group   = %{ $regex_groups{$code} };
@@ -228,6 +184,49 @@ sub _create_controls {
 		}
 	}
 
+	# Optionally toggle the visibility of the description field
+	$self->{description_checkbox} = Wx::CheckBox->new(
+		$self,
+		-1,
+		Wx::gettext('Show &Description'),
+	);
+
+	# Describe-the-regex text field
+	$self->{description_text} = Wx::TextCtrl->new(
+		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
+		Wx::wxTE_MULTILINE | Wx::wxNO_FULL_REPAINT_ON_RESIZE
+	);
+
+	# Description is hidden by default
+	$self->{description_text}->Hide;
+
+	# Replace regex text field
+	my $replace_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Replace text with:') );
+	$self->{replace} = Wx::TextCtrl->new(
+		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
+		Wx::wxTE_MULTILINE | Wx::wxNO_FULL_REPAINT_ON_RESIZE
+	);
+
+	# Original input text field
+	my $original_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Original text:') );
+	$self->{original_text} = Wx::TextCtrl->new(
+		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
+		Wx::wxTE_MULTILINE | Wx::wxNO_FULL_REPAINT_ON_RESIZE
+	);
+
+	# Matched readonly text field
+	my $matched_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Matched text:') );
+	$self->{matched_text} = Wx::RichTextCtrl->new(
+		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
+		Wx::wxRE_MULTILINE | Wx::wxRE_READONLY | Wx::wxWANTS_CHARS # Otherwise arrows will not work on win32
+	);
+
+	# Result from replace text field
+	my $result_label = Wx::StaticText->new( $self, -1, Wx::gettext('&Result from replace:') );
+	$self->{result_text} = Wx::RichTextCtrl->new(
+		$self, -1, '', Wx::wxDefaultPosition, Wx::wxDefaultSize,
+		Wx::wxRE_MULTILINE | Wx::wxRE_READONLY | Wx::wxWANTS_CHARS # Otherwise arrows will not work on win32
+	);
 
 	# Insert regex into current document button_name
 	$self->{insert_button} = Wx::Button->new(
@@ -245,6 +244,18 @@ sub _create_controls {
 	$buttons->Add( $self->{close_button},  0, Wx::wxALL, 1 );
 	$buttons->AddStretchSpacer;
 
+	# Modifiers
+	my %m = $self->_modifiers();
+	foreach my $name ( $self->_modifier_keys() ) {
+		$self->{$name} = Wx::CheckBox->new(
+			$self,
+			-1,
+			$m{$name}{name},
+		);
+
+		$self->{$name}->SetToolTip( Wx::ToolTip->new( $m{$name}{tooltip} ) );
+	}
+
 	# Dialog Layout
 
 	my $modifiers = Wx::BoxSizer->new(Wx::wxHORIZONTAL);
@@ -274,10 +285,10 @@ sub _create_controls {
 	my $left = Wx::BoxSizer->new(Wx::wxVERTICAL);
 	$left->Add( $modifiers, 0, Wx::wxALL | Wx::wxEXPAND, 2 );
 	$left->AddSpacer(5);
-	$left->Add( $regex_label,              0, Wx::wxALL | Wx::wxEXPAND, 1 );
-	$left->Add( $combined,                 0, Wx::wxALL | Wx::wxEXPAND, 2 );
-	$left->Add( $description_label,        0, Wx::wxALL | Wx::wxEXPAND, 1 );
-	$left->Add( $self->{description_text}, 2, Wx::wxALL | Wx::wxEXPAND, 1 );
+	$left->Add( $regex_label,                  0, Wx::wxALL | Wx::wxEXPAND, 1 );
+	$left->Add( $combined,                     0, Wx::wxALL | Wx::wxEXPAND, 2 );
+	$left->Add( $self->{description_checkbox}, 0, Wx::wxALL | Wx::wxEXPAND, 1 );
+	$left->Add( $self->{description_text},     2, Wx::wxALL | Wx::wxEXPAND, 1 );
 
 	$left->Add( $replace_label,   0, Wx::wxALL | Wx::wxEXPAND, 1 );
 	$left->Add( $self->{replace}, 1, Wx::wxALL | Wx::wxEXPAND, 1 );
@@ -303,6 +314,14 @@ sub _bind_events {
 		$self->{regex},
 		sub { $_[0]->run; },
 	);
+	Wx::Event::EVT_KEY_DOWN(
+		$self,
+		sub {
+			my ($key_event) = $_[1];
+			$self->Hide if $key_event->GetKeyCode == Wx::WXK_ESCAPE;
+			return;
+		}
+	);
 	Wx::Event::EVT_TEXT(
 		$self,
 		$self->{replace},
@@ -315,8 +334,7 @@ sub _bind_events {
 	);
 
 	# Modifiers
-	my %modifiers = $self->_modifiers();
-	foreach my $name ( keys %modifiers ) {
+	foreach my $name ( $self->_modifier_keys ) {
 		Wx::Event::EVT_CHECKBOX(
 			$self,
 			$self->{$name},
@@ -326,32 +344,97 @@ sub _bind_events {
 		);
 	}
 
+	# Description checkbox
+	Wx::Event::EVT_CHECKBOX(
+		$self,
+		$self->{description_checkbox},
+		sub {
+
+			# toggles the visibility of the description field
+			$self->{description_text}->Show( $self->{description_checkbox}->IsChecked );
+			$self->{sizer}->Layout;
+		},
+	);
+
+	Wx::Event::EVT_KEY_DOWN(
+		$self->{matched_text},
+		sub {
+			my ($key_event) = $_[1];
+			$self->Hide if $key_event->GetKeyCode == Wx::WXK_ESCAPE;
+			return;
+		}
+	);
+
+	Wx::Event::EVT_KEY_DOWN(
+		$self->{result_text},
+		sub {
+			my ($key_event) = $_[1];
+			$self->Hide if $key_event->GetKeyCode == Wx::WXK_ESCAPE;
+			return;
+		}
+	);
+
 	Wx::Event::EVT_BUTTON(
 		$self,
 		$self->{insert_button},
-		sub {
-			my $self = shift;
-			my $editor = $self->current->editor or return;
-			$editor->InsertText( $editor->GetCurrentPos, $self->{regex}->GetValue );
-		},
+		sub { shift->_insert_regex; },
 	);
 }
 
+#
+# A private method that inserts the current regex into the current document
+#
+sub _insert_regex {
+	my $self = shift;
+
+	my $match_part   = $self->{regex}->GetValue;
+	my $replace_part = $self->{replace}->GetValue;
+
+	my $modifiers = $self->_get_modifier_settings;
+
+	my $editor = $self->current->editor or return;
+	$editor->InsertText( $editor->GetCurrentPos, "s/$match_part/$replace_part/$modifiers" );
+
+	return;
+}
+
 
 #
 # A private method that returns a hash of regex modifiers
 #
 sub _modifiers {
-	my $self = shift;
 	return (
-		ignore_case => { mod => 'i', name => sprintf( Wx::gettext('&Ignore case (%s)'), 'i' ) },
-		single_line => { mod => 's', name => sprintf( Wx::gettext('&Single-line (%s)'), 's' ) },
-		multi_line  => { mod => 'm', name => sprintf( Wx::gettext('&Multi-line (%s)'),  'm' ) },
-		extended    => { mod => 'x', name => sprintf( Wx::gettext('&Extended (%s)'),    'x' ) },
-		global      => { mod => 'g', name => sprintf( Wx::gettext('&Global (%s)'),      'g' ) },
+		ignore_case => {
+			mod     => 'i', name => Wx::gettext('Ignore case (&i)'),
+			tooltip => Wx::gettext('Case-insensitive matching')
+		},
+		single_line => {
+			mod     => 's', name => Wx::gettext('Single-line (&s)'),
+			tooltip => Wx::gettext('"." also matches newline')
+		},
+		multi_line => {
+			mod => 'm', name => Wx::gettext('Multi-line (&m)'),
+			tooltip => Wx::gettext('"^" and "$" match the start and end of any line inside the string')
+		},
+		extended => {
+			mod => 'x', name => Wx::gettext('Extended (&x)'),
+			tooltip =>
+				Wx::gettext('Extended regular expressions allow free formatting (whitespace is ignored) and comments')
+		},
+		global => {
+			mod     => 'g', name => Wx::gettext('Global (&g)'),
+			tooltip => Wx::gettext('Replace all occurrences of the pattern')
+		},
 	);
 }
 
+#
+# returns the regex modifier keys in the order they appear in the GUI
+#
+sub _modifier_keys {
+	return qw{ ignore_case single_line multi_line extended	global};
+}
+
 
 # -- public methods
 
@@ -374,8 +457,8 @@ sub show {
 			$self->{regex}->ChangeValue('\w+');
 		}
 
-		$self->{replace}->ChangeValue("Baz");
-		$self->{original_text}->SetValue("Foo Bar");
+		$self->{replace}->ChangeValue('Baz');
+		$self->{original_text}->SetValue('Foo Bar');
 
 		$self->Show;
 	}
@@ -386,11 +469,17 @@ sub show {
 }
 
 #
-# Private method to dump the regular expression
-# description as text
+# Private method to dump the regular expression description as text
 #
 sub _dump_regex {
+	if ( scalar @_ == 2 ) {
+		my ( $self, $regex ) = @_;
+		require PPIx::Regexp;
+		return $self->_dump_regex( PPIx::Regexp->new("/$regex/"), '', 0 );
+	}
+
 	my ( $self, $parent, $str, $level ) = @_;
+
 	$str   = '' unless $str;
 	$level = 0  unless $level;
 	my @children = $parent->isa('PPIx::Regexp::Node') ? $parent->children : ();
@@ -398,7 +487,7 @@ sub _dump_regex {
 		next if $child->content eq '';
 		my $class_name = $child->class;
 		$class_name =~ s/PPIx::Regexp:://;
-		$str .= ( " " x ( $level * 4 ) ) . $class_name . "     (" . $child->content . ")\n";
+		$str .= ( ' ' x ( $level * 4 ) ) . $class_name . '     (' . $child->content . ")\n";
 		$str = $self->_dump_regex( $child, $str, $level + 1 );
 	}
 	return $str;
@@ -428,26 +517,75 @@ sub _parse_regex_elements {
 	return @array;
 }
 
+#
+# Returns the user input data of the dialog as a hashref
+#
+sub get_data {
+	my $self = shift;
+
+	my %data = (
+		text => {
+			regex         => $self->{regex}->GetValue,
+			replace       => $self->{replace}->GetValue,
+			original_text => $self->{original_text}->GetValue,
+		},
+		modifiers => [ $self->_get_modifier_settings ],
+	);
+
+	return \%data;
+}
+
+#
+# Sets the user input data of the dialog given a hashref containing the results of get_data
+#
+sub set_data {
+	my ( $self, $data_ref ) = @_;
+
+	foreach my $text_field ( keys %{ $data_ref->{text} } ) {
+		$self->{$text_field}->SetValue( $data_ref->{text}->{$text_field} );
+	}
+
+	my $modifier_string = $data_ref->{modifiers}->[0];
+	my %modifiers       = $self->_modifiers();
+	foreach my $name ( keys %modifiers ) {
+		$self->{$name}->SetValue(1) if $modifier_string =~ s/$modifiers{$name}{mod}//;
+	}
+
+	return;
+}
+
+#
+# Private method to get the modifier settings as two strings
+# the first strings returns the active modifiers, the second the
+# inactive ones
+#
+sub _get_modifier_settings {
+	my $self = shift;
+
+	my $active_modifiers   = '';
+	my $inactive_modifiers = '';
+	my %modifiers          = $self->_modifiers();
+	foreach my $name ( keys %modifiers ) {
+		if ( $self->{$name}->IsChecked ) {
+			$active_modifiers .= $modifiers{$name}{mod};
+		} else {
+			$inactive_modifiers .= $modifiers{$name}{mod};
+		}
+	}
+
+	return ( $active_modifiers, $inactive_modifiers );
+}
+
 sub run {
 	my $self = shift;
 
-	my $regex = $self->{regex}->GetRange( 0, $self->{regex}->GetLastPosition );
-	my $original_text = $self->{original_text}->GetRange( 0, $self->{original_text}->GetLastPosition );
-	my $replace = $self->{replace}->GetRange( 0, $self->{replace}->GetLastPosition );
-	my $result_text = $original_text;
+	my $regex         = $self->{regex}->GetValue;
+	my $original_text = $self->{original_text}->GetValue;
+	my $replace       = $self->{replace}->GetValue;
+	my $result_text   = $original_text;
 
-
-	my $start     = '';
-	my $end       = '';
-	my %modifiers = $self->_modifiers();
-	foreach my $name ( keys %modifiers ) {
-		if ( $self->{$name}->IsChecked ) {
-			$start .= $modifiers{$name}{mod};
-		} else {
-			$end .= $modifiers{$name}{mod};
-		}
-	}
-	my $xism = "$start-$end";
+	my ( $active, $inactive ) = $self->_get_modifier_settings;
+	my $xism = "$active-$inactive";
 
 	$self->{matched_text}->Clear;
 	$self->{result_text}->Clear;
@@ -476,14 +614,14 @@ sub run {
 	};
 	if ($@) {
 		$self->{matched_text}->BeginTextColour(Wx::wxRED);
-		$self->{matched_text}->SetValue( sprintf( Wx::gettext("Match failure in %s:  %s"), $regex, $@ ) );
+		$self->{matched_text}->SetValue( sprintf( Wx::gettext('Match failure in %s:  %s'), $regex, $@ ) );
 		$self->{matched_text}->EndTextColour;
 		return;
 	}
 
 	if ($warning) {
 		$self->{matched_text}->BeginTextColour(Wx::wxRED);
-		$self->{matched_text}->SetValue( sprintf( Wx::gettext("Match warning in %s:  %s"), $regex, $warning ) );
+		$self->{matched_text}->SetValue( sprintf( Wx::gettext('Match warning in %s:  %s'), $regex, $warning ) );
 		$self->{matched_text}->EndTextColour;
 		return;
 	}
@@ -494,8 +632,10 @@ sub run {
 		foreach my $char (@chars) {
 			if ( $pos == $match_start ) {
 				$self->{matched_text}->BeginTextColour(Wx::wxRED);
+				$self->{matched_text}->BeginUnderline;
 			} elsif ( $pos == $match_end ) {
 				$self->{matched_text}->EndTextColour;
+				$self->{matched_text}->EndUnderline;
 			}
 			$self->{matched_text}->AppendText($char);
 			$pos++;
@@ -506,14 +646,11 @@ sub run {
 		$self->{matched_text}->EndTextColour;
 	}
 
-	eval {
-		if ( $self->{global}->IsChecked )
-		{
-			$result_text =~ s{(?$xism:$regex)}{$replace}g;
-		} else {
-			$result_text =~ s{(?$xism:$regex)}{$replace};
-		}
-	};
+	if ( $self->{global}->IsChecked ) {
+		eval "\$result_text =~ s{(?$xism:$regex)}{$replace}g";
+	} else {
+		eval "\$result_text =~ s{(?$xism:$regex)}{$replace}";
+	}
 	if ($@) {
 		$self->{result_text}->BeginTextColour(Wx::wxRED);
 		$self->{result_text}->AppendText( sprintf( Wx::gettext('Replace failure in %s:  %s'), $regex, $@ ) );
@@ -527,9 +664,7 @@ sub run {
 
 	$self->{matched_text}->EndTextColour;
 
-	require PPIx::Regexp;
-	my $regexp = PPIx::Regexp->new("/$regex/");
-	$self->{description_text}->SetValue( $self->_dump_regex($regexp) );
+	$self->{description_text}->SetValue( $self->_dump_regex($regex) ) if $self->{description_text}->IsShown;
 
 	#	$self->{regex}->Clear;
 	#	my @elements = _parse_regex_elements;
@@ -621,8 +756,3 @@ This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5 itself.
 
 =cut
-
-# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
-# LICENSE
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl 5 itself.
