@@ -4,13 +4,15 @@ use 5.008;
 use strict;
 use warnings;
 use YAML::Tiny                ();
+use Time::HiRes               ();
 use Padre::Constant           ();
 use Padre::Util               ();
 use Padre::Current            ();
 use Padre::Wx                 ();
 use Padre::Wx::FileDropTarget ();
+use Padre::Logger;
 
-our $VERSION = '0.50';
+our $VERSION = '0.76';
 our @ISA     = 'Wx::StyledTextCtrl';
 
 # End-Of-Line modes:
@@ -28,6 +30,8 @@ our %mode = (
 # mapping for mime-type to the style name in the share/styles/default.yml file
 our %MIME_STYLE = (
 	'application/x-perl' => 'perl',
+	'application/x-psgi' => 'perl',
+	'text/x-perlxs'      => 'xs',   # should be in the plugin...
 	'text/x-patch'       => 'diff',
 	'text/x-makefile'    => 'make',
 	'text/x-yaml'        => 'yaml',
@@ -35,6 +39,14 @@ our %MIME_STYLE = (
 	'application/x-php'  => 'perl', # temporary solution
 );
 
+# Karl
+# these are the allowed braces for brace highlighting and brace matching
+# this has to be subset of  ( ) [ ] { } < > since we use the scintilla
+# Brace* methods
+# always altern opening and starting braces in the constant
+my $BRACES               = '{}[]()';
+my $STC_INVALID_POSITION = Wx::wxSTC_INVALID_POSITION;
+
 my $data;
 my $data_name;
 my $data_private;
@@ -42,14 +54,22 @@ my $width;
 my $Clipboard_Old = '';
 
 sub new {
-	my $class    = shift;
-	my $notebook = shift;
+	my $class  = shift;
+	my $parent = shift;
+
+	# NOTE: This hack is only here because the Preferences dialog uses
+	# an editor object for their style preview thingy.
+	my $main = $parent;
+	while ( not $main->isa('Padre::Wx::Main') ) {
+		$main = $main->GetParent;
+	}
 
 	# Create the underlying Wx object
-	my $self   = $class->SUPER::new($notebook);
-	my $config = $self->main->ide->config;
+	my $lock = $main->lock( 'UPDATE', 'refresh_windowlist' );
+	my $self = $class->SUPER::new($parent);
 
-	# TODO: Make this suck less
+	# TO DO: Make this suck less
+	my $config = $main->config;
 	$data = data( $config->editor_style );
 
 	# Set the code margins a little larger than the default.
@@ -70,6 +90,8 @@ sub new {
 	Wx::Event::EVT_CHAR( $self, \&on_char );
 	Wx::Event::EVT_SET_FOCUS( $self, \&on_focus );
 	Wx::Event::EVT_MIDDLE_UP( $self, \&on_middle_up );
+	Wx::Event::EVT_MOTION( $self, \&on_mouse_moving );
+	Wx::Event::EVT_MOUSEWHEEL( $self, \&on_mousewheel );
 
 	# Smart highlighting:
 	# Selecting a word or small block of text causes all other occurrences to be highlighted
@@ -95,13 +117,67 @@ sub new {
 
 	$self->SetDropTarget( Padre::Wx::FileDropTarget->new( $self->main ) );
 
+	# Disable CTRL keypad -/+. These seem to emit wrong scan codes
+	# on some laptop keyboards. (e.g. CTRL-Caps lock is the same as CTRL -)
+	# Please see bug #790
+	$self->CmdKeyClear( Wx::wxSTC_KEY_SUBTRACT, Wx::wxSTC_SCMOD_CTRL );
+	$self->CmdKeyClear( Wx::wxSTC_KEY_ADD,      Wx::wxSTC_SCMOD_CTRL );
+
+	my $green = Wx::Colour->new("green");
+	my $red   = Wx::Colour->new("red");
+	my $blue  = Wx::Colour->new("blue");
+
+	#NOTE: DO NOT USE "orange" string since it is actually red on win32
+	my $orange = Wx::Colour->new( 255, 165, 0 );
+
+	$self->MarkerDefine(
+		Padre::Wx::MarkError(),
+		Wx::wxSTC_MARK_SMALLRECT,
+		$red,
+		$red,
+	);
+	$self->MarkerDefine(
+		Padre::Wx::MarkWarn(),
+		Wx::wxSTC_MARK_SMALLRECT,
+		$orange,
+		$orange,
+	);
+	$self->MarkerDefine(
+		Padre::Wx::MarkLocation(),
+		Wx::wxSTC_MARK_SMALLRECT,
+		$green,
+		$green,
+	);
+	$self->MarkerDefine(
+		Padre::Wx::MarkBreakpoint(),
+		Wx::wxSTC_MARK_SMALLRECT,
+		$blue,
+		$blue,
+	);
+
 	return $self;
 }
 
 sub main {
-	return $_[0]->GetGrandParent;
+	$_[0]->GetGrandParent;
 }
 
+# convenience accessor method (and to ensure consistency)
+# return the Padre::Config instance
+sub config {
+	$_[0]->main->config;
+}
+
+# convenience methods
+# return the character at a given position as a perl string
+sub get_character_at {
+	my ( $self, $pos ) = @_;
+	return chr( $self->GetCharAt($pos) );
+}
+
+
+
+
 sub data {
 	my $name    = shift;
 	my $private = shift;
@@ -128,22 +204,24 @@ sub data {
 	return $data;
 }
 
-sub error { # Error Message
+# Error Message
+sub error {
 	my $self = shift;
 	my $text = shift;
 	Wx::MessageBox(
-		$text,    Wx::gettext("Error"),
-		Wx::wxOK, $self->main
+		$text,
+		Wx::gettext("Error"),
+		Wx::wxOK,
+		$self->main
 	);
-
 }
 
-# most of this should be read from some external files
+# Most of this should be read from some external files
 # but for now we use this if statement
 sub padre_setup {
 	my $self = shift;
 
-	Padre::Util::debug("before setting the lexer");
+	TRACE("before setting the lexer") if DEBUG;
 	$self->SetLexer( $self->{Document}->lexer );
 
 	# the next line will change the ESC key to cut the current selection
@@ -154,9 +232,10 @@ sub padre_setup {
 	# and Wx::wxUNICODE or wxUSE_UNICODE should be on
 	$self->SetCodePage(65001);
 
-	my $mimetype = $self->{Document}->get_mimetype || 'text/plain';
+	my $mimetype = $self->{Document}->mimetype || 'text/plain';
 	if ( $MIME_STYLE{$mimetype} ) {
 		$self->padre_setup_style( $MIME_STYLE{$mimetype} );
+
 	} elsif ( $mimetype eq 'text/plain' ) {
 		$self->padre_setup_plain;
 		my $filename = $self->{Document}->filename || q{};
@@ -166,6 +245,7 @@ sub padre_setup {
 			# re-setup if file extension is .conf
 			$self->padre_setup_style('conf') if $ext eq 'conf';
 		}
+
 	} elsif ($mimetype) {
 
 		# setup some default coloring
@@ -181,9 +261,7 @@ sub padre_setup {
 	return;
 }
 
-#
-# Called a key is released in the editor
-#
+# Called when a key is released in the editor
 sub on_key_up {
 	my ( $self, $event ) = @_;
 
@@ -192,19 +270,30 @@ sub on_key_up {
 		my $line = $self->GetLine( $self->GetCurrentLine() );
 		if ( $line !~ /^\s*$/ ) {
 
-			# Only cut on non-black lines
+			# Only cut on non-blank lines
 			$self->CmdKeyExecute(Wx::wxSTC_CMD_LINECUT);
 		} else {
 
 			# Otherwise delete the line
 			$self->CmdKeyExecute(Wx::wxSTC_CMD_LINEDELETE);
 		}
+		$event->Skip(0); # done processing this nothing more to do
+		return;
 	}
 
 	# Apply smart highlighting when the shift key is down
 	if ( $self->main->ide->config->editor_smart_highlight_enable && $event->ShiftDown ) {
 		$self->on_smart_highlight_begin($event);
 	}
+
+	# Doc specific processing
+	my $doc = $self->{Document};
+	if ( $doc->can('event_key_up') ) {
+		$doc->event_key_up( $self, $event );
+	}
+
+	$event->Skip(1); # we need to keep processing this event
+
 }
 
 sub padre_setup_plain {
@@ -267,38 +356,61 @@ sub padre_setup_style {
 }
 
 sub setup_style_from_config {
-	my ( $self, $name ) = @_;
+	my $self   = shift;
+	my $name   = shift;
+	my $style  = $data->{$name};
+	my $colors = $style->{colors};
 
-	foreach my $k ( keys %{ $data->{$name}->{colors} } ) {
-		my $f = 'Wx::' . $k;
-		if ( $k =~ /^PADRE_/ ) {
-			$f = 'Padre::Constant::' . $k;
-		}
-		no strict "refs"; ## no critic
-		my $v = eval { $f->() };
-		if ($@) {
-			warn "invalid key '$k'\n";
-			next;
+	# The selection background (if applicable)
+	# (The Scintilla official selection background colour is cc0000)
+	if ( $style->{selection_background} ) {
+		$self->SetSelBackground( 1, _color( $style->{selection_background} ) );
+	}
+	if ( $style->{selection_foreground} ) {
+		$self->SetSelForeground( 1, _color( $style->{selection_foreground} ) );
+	}
+
+	# Set the styles
+	foreach my $k ( keys %$colors ) {
+		my $v;
+
+		# allow for plain numbers
+		if ( $k =~ /^\d+$/ ) {
+			$v = $k;
 		}
 
-		my $colors = $data->{$name}->{colors}->{$k};
-		if ( exists $colors->{foreground} ) {
-			$self->StyleSetForeground( $f->(), _color( $colors->{foreground} ) );
+		# but normally, we have Wx:: or PADRE_ constants
+		else {
+			my $f = 'Wx::' . $k;
+			if ( $k =~ /^PADRE_/ ) {
+				$f = 'Padre::Constant::' . $k;
+			}
+			no strict "refs";
+			$v = eval { $f->() };
+			if ($@) {
+				warn "invalid key '$k'\n";
+				next;
+			}
 		}
-		if ( exists $colors->{background} ) {
-			$self->StyleSetBackground( $f->(), _color( $colors->{background} ) );
+
+		my $color = $data->{$name}->{colors}->{$k};
+		if ( exists $color->{foreground} ) {
+			$self->StyleSetForeground( $v, _color( $color->{foreground} ) );
 		}
-		if ( exists $colors->{bold} ) {
-			$self->StyleSetBold( $f->(), $colors->{bold} );
+		if ( exists $color->{background} ) {
+			$self->StyleSetBackground( $v, _color( $color->{background} ) );
 		}
-		if ( exists $colors->{italics} ) {
-			$self->StyleSetItalic( $f->(), $colors->{italic} );
+		if ( exists $color->{bold} ) {
+			$self->StyleSetBold( $v, $color->{bold} );
 		}
-		if ( exists $colors->{eolfilled} ) {
-			$self->StyleSetEOLFilled( $f->(), $colors->{eolfilled} );
+		if ( exists $color->{italics} ) {
+			$self->StyleSetItalic( $v, $color->{italic} );
 		}
-		if ( exists $colors->{underlined} ) {
-			$self->StyleSetUnderline( $f->(), $colors->{underline} );
+		if ( exists $color->{eolfilled} ) {
+			$self->StyleSetEOLFilled( $v, $color->{eolfilled} );
+		}
+		if ( exists $color->{underlined} ) {
+			$self->StyleSetUnderline( $v, $color->{underline} );
 		}
 	}
 }
@@ -318,32 +430,203 @@ sub _color {
 	return Wx::Colour->new(@c);
 }
 
+
+
+
+
+=head2 get_brace_info
+
+Look at a given position in the editor if there is a brace (according to the 
+setting editor_braces) before or after, and return the information about the context
+It always look first at the character after the position.
+
+	Params:
+		pos - the cursor position in the editor [defaults to cursor position) : int
+		
+	Return:
+		undef if no brace, otherwise [brace, actual_pos, is_after, is_opening]
+		where:
+			brace - the brace char at actual_pos
+			actual_pos - the actual position where the brace has been found
+			is_after - true iff the brace is after the cursor : boolean
+			is_opening - true iff only the brace is an opening one
+			
+	Examples:
+
+		|{} => should find the { : [0,{,1,1] 
+		{|} => should find the } : [1,},1,0]
+		{| } => should find the { : [0,{,0,1]
+
+=cut
+
+
+sub get_brace_info {
+	my ( $self, $pos ) = @_;
+	$pos = $self->GetCurrentPos unless defined $pos;
+
+	# try the after position first (default one for BraceMatch)
+	my $is_after = 1;
+	my $brace    = $self->get_character_at($pos);
+	my $is_brace = $self->get_brace_type($brace);
+	if ( !$is_brace && $pos > 0 ) { # try the before position
+		$brace    = $self->get_character_at( --$pos );
+		$is_brace = $self->get_brace_type($brace) or return undef;
+		$is_after = 0;
+	}
+	my $is_opening = $is_brace % 2; # odd values are opening
+	return [ $pos, $brace, $is_after, $is_opening ];
+}
+
+
+=head2 get_brace_type
+
+Tell if a character is a brace, and if it is an opening or a closing one
+
+	Params:
+		char - a character : string
+		
+	Return:
+		int : 0 if this is not a brace, an odd value if it is an opening brace and an even
+		one for a closing brace
+
+=cut
+
+my %_cached_braces;
+
+sub get_brace_type {
+	my ( $self, $char ) = @_;
+	unless (%_cached_braces) {
+		my $i = 1; # start from one so that all values are true
+		$_cached_braces{$_} = $i++ foreach ( split //, $BRACES );
+	}
+	my $v = $_cached_braces{$char} or return 0;
+	return $v;
+}
+
+
+
+# some uncorrect behaviour (| is the cursor)
+# {} : never highlighted
+# { } : always correct
+#
+#
+
+sub apply_style {
+	my ( $self, $style_info ) = @_;
+	my %previous_style = %$style_info;
+	$previous_style{style} = $self->GetStyleAt( $style_info->{start} );
+
+	$self->StartStyling( $style_info->{start}, 0xFF );
+	$self->SetStyling( $style_info->{len}, $style_info->{style} );
+
+	return \%previous_style;
+}
+
+
+my $previous_expr_hiliting_style;
+
 sub highlight_braces {
 	my ($self) = @_;
 
-	$self->BraceHighlight( -1, -1 ); # Wx::wxSTC_INVALID_POSITION
-	my $pos1 = $self->GetCurrentPos;
-	my $chr  = chr( $self->GetCharAt($pos1) );
+	my $expression_highlighting = $self->config->editor_brace_expression_highlighting;
 
-	my @braces = ( '{', '}', '(', ')', '[', ']' );
-	if ( not grep { $chr eq $_ } @braces ) {
-		if ( $pos1 > 0 ) {
-			$pos1--;
-			$chr = chr( $self->GetCharAt($pos1) );
-			return unless grep { $chr eq $_ } @braces;
-		}
+	# remove current highlighting if any
+	$self->BraceHighlight( $STC_INVALID_POSITION, $STC_INVALID_POSITION );
+	if ($previous_expr_hiliting_style) {
+		$self->apply_style($previous_expr_hiliting_style);
+		$previous_expr_hiliting_style = undef;
 	}
 
-	my $pos2 = $self->BraceMatch($pos1);
-	return if abs( $pos1 - $pos2 ) < 2;
+	my $pos1          = $self->GetCurrentPos;
+	my $info1         = $self->get_brace_info($pos1) or return;
+	my ($actual_pos1) = @$info1;
 
-	return if $pos2 == -1; #Wx::wxSTC_INVALID_POSITION  #????
+	my $actual_pos2 = $self->BraceMatch($actual_pos1);
+
+	#	return if abs( $pos1 - $pos2 ) < 2;
+
+	return if $actual_pos2 == $STC_INVALID_POSITION; #Wx::wxSTC_INVALID_POSITION  #????
+
+	$self->BraceHighlight( $actual_pos1, $actual_pos2 );
+
+	if ($expression_highlighting) {
+		my $pos2 = $self->find_matching_brace($pos1) or return;
+		my %style = (
+			start => $pos1 < $pos2 ? $pos1 : $pos2,
+			len => abs( $pos1 - $pos2 ), style => Wx::wxSTC_STYLE_DEFAULT
+		);
+		$previous_expr_hiliting_style = $self->apply_style( \%style );
+	}
 
-	$self->BraceHighlight( $pos1, $pos2 );
 
 	return;
 }
 
+
+=head2 find_matching_brace
+
+Find the position of to the matching brace if any. If the cursor is inside the braces the destination 
+will be inside too, same it is outside.
+
+	Params:
+		pos - the cursor position in the editor [defaults to cursor position) : int
+		
+	Return:
+		matching_pos - the matching position, or undef if none
+
+=cut
+
+sub find_matching_brace {
+	my ( $self, $pos ) = @_;
+	$pos = $self->GetCurrentPos unless defined $pos;
+	my $info1 = $self->get_brace_info($pos) or return;
+	my ( $actual_pos1, $brace, $is_after, $is_opening ) = @$info1;
+
+	my $actual_pos2 = $self->BraceMatch($actual_pos1);
+	return if $actual_pos2 == $STC_INVALID_POSITION;
+	$actual_pos2++ if $is_after; # ensure is stays inside if origin is inside, same four outside
+	return $actual_pos2;
+}
+
+
+=head2 goto_matching_brace
+
+Move the cursor to the matching brace if any. If the cursor is inside the braces the destination 
+will be inside too, same it is outside.
+
+	Params:
+		pos - the cursor position in the editor [defaults to cursor position) : int
+		
+
+=cut
+
+sub goto_matching_brace {
+	my ( $self, $pos ) = @_;
+	my $pos2 = $self->find_matching_brace($pos) or return;
+	$self->GotoPos($pos2);
+}
+
+=head2 select_to_matching_brace
+
+Select to the matching opening or closing brace. If the cursor is inside the braces the destination 
+will be inside too, same it is outside.
+
+	Params:
+		pos - the cursor position in the editor [defaults to cursor position) : int
+		
+
+
+=cut
+
+sub select_to_matching_brace {
+	my ( $self, $pos ) = @_;
+	$pos = $self->GetCurrentPos unless defined $pos;
+	my $pos2 = $self->find_matching_brace($pos) or return;
+	my $start = ( $pos < $pos2 ) ? $self->GetSelectionStart() : $self->GetSelectionEnd();
+	$self->SetSelection( $start, $pos2 );
+
+}
+
 # currently if there are 9 lines we set the margin to 1 width and then
 # if another line is added it is not seen well.
 # actually I added some improvement allowing a 50% growth in the file
@@ -417,7 +700,7 @@ sub show_folding {
 					my $line_clicked  = $editor->LineFromPosition( $event->GetPosition() );
 					my $level_clicked = $editor->GetFoldLevel($line_clicked);
 
-					# TODO check this (cf. ~/contrib/samples/stc/edit.cpp from wxWidgets)
+					# TO DO check this (cf. ~/contrib/samples/stc/edit.cpp from wxWidgets)
 					#if ( $level_clicked && wxSTC_FOLDLEVELHEADERFLAG) > 0) {
 					$editor->ToggleFold($line_clicked);
 
@@ -502,7 +785,7 @@ sub show_calltip {
 # 3) while doing all this, respect the current (sadly global) indentation settings
 # For auto-de-indentation (i.e. closing brace), we remove one level of indentation
 # instead.
-# FIXME/TODO: needs some refactoring
+# FIX ME/TO DO: needs some refactoring
 sub autoindent {
 	my ( $self, $mode ) = @_;
 
@@ -618,7 +901,7 @@ sub _auto_deindent {
 	# this is if the line matches "blahblahSomeText}".
 	elsif ( $config->editor_autoindent eq 'deep' and $content =~ /\}\s*$/ ) {
 
-		# TODO: What should happen in this case?
+		# TO DO: What should happen in this case?
 	}
 
 	return;
@@ -672,56 +955,54 @@ sub unfold_all {
 	return;
 }
 
-# when the focus is received by the editor
+# When the focus is received by the editor
 sub on_focus {
-	my ( $self, $event ) = @_;
-	my $doc = Padre::Current->document;
+	my $self     = shift;
+	my $event    = shift;
+	my $main     = $self->main;
+	my $document = $main->current->document;
+	TRACE( "Focus received file: " . ( $document->filename || '' ) ) if DEBUG;
 
-	Padre::Util::debug( "Focus received file: " . ( $doc->filename || '' ) );
+	# NOTE: The editor focus event fires a LOT, even for trivial things
+	# like changing focus to another application and immediately back again,
+	# or switching between tools in Padre.
+	# Don't do any refreshing here, it is an excessive waste of resources.
+	# Instead, put them in the events that ACTUALLY change application state.
 
-	my $main = $self->main;
-
-	# to show/hide the document specific Perl menu
-	$main->refresh_menu;
-
-	$main->update_directory;
-
-	# TODO
-	# this is called even if the mouse is moved away from padre and back again
+	# TO DO
+	# This is called even if the mouse is moved away from padre and back again
 	# we should restrict some of the updates to cases when we switch from one file to
 	# another
-
 	if ( $self->needs_manual_colorize ) {
-		Padre::Util::debug("needs_manual_colorize");
+		TRACE("needs_manual_colorize") if DEBUG;
+		my $lock  = $main->lock('UPDATE');
 		my $lexer = $self->GetLexer;
 		if ( $lexer == Wx::wxSTC_LEX_CONTAINER ) {
-			$doc->colorize;
+			$document->colorize;
 		} else {
-			$doc->remove_color;
+			$document->remove_color;
 			$self->Colourise( 0, $self->GetLength );
 		}
 		$self->needs_manual_colorize(0);
 	} else {
-		Padre::Util::debug("no need to colorize");
+		TRACE("no need to colorize") if DEBUG;
 	}
 
-	$event->Skip(1); # so the cursor will show up
+	# NOTE: This is so the cursor will show up
+	$event->Skip(1);
+
 	return;
 }
 
 sub on_char {
-	my ( $self, $event ) = @_;
-
-	my $doc = $self->{Document};
-	if ( $doc->can('event_on_char') ) {
-		$doc->event_on_char( $self, $event );
+	my $self     = shift;
+	my $event    = shift;
+	my $document = $self->{Document};
+	if ( $document->can('event_on_char') ) {
+		$document->event_on_char( $self, $event );
 	}
 
-	if ( $self->main->ide->{has_Time_HiRes} ) {
-		$doc->{last_char_time} = Time::HiRes::time();
-	} else {
-		$doc->{last_char_time} = time;
-	}
+	$document->{last_char_time} = Time::HiRes::time();
 
 	$event->Skip;
 	return;
@@ -780,7 +1061,7 @@ sub on_smart_highlight_begin {
 	if ( scalar @{ $self->{styles} } > 1 ) {
 		foreach my $style ( @{ $self->{styles} } ) {
 			$self->StartStyling( $style->{start}, 0xFF );
-			$self->SetStyling( $style->{len}, 32 );
+			$self->SetStyling( $style->{len}, Wx::wxSTC_STYLE_DEFAULT );
 		}
 	}
 
@@ -796,12 +1077,18 @@ sub on_smart_highlight_end {
 sub on_left_up {
 	my ( $self, $event ) = @_;
 
+	my $config = $self->main->ide->config;
+
 	my $text = $self->GetSelectedText;
 	if ( Padre::Constant::WXGTK and defined $text and $text ne '' ) {
 
 		# Only on X11 based platforms
 		#		Wx::wxTheClipboard->UsePrimarySelection(1);
-		$self->put_text_to_clipboard($text);
+		if ( $config->mid_button_paste ) {
+			$self->put_text_to_clipboard( $text, 1 );
+		} else {
+			$self->put_text_to_clipboard($text);
+		}
 
 		#		Wx::wxTheClipboard->UsePrimarySelection(0);
 	}
@@ -815,24 +1102,56 @@ sub on_left_up {
 	return;
 }
 
+sub on_mouse_moving {
+	my ( $self, $event ) = @_;
+
+	if ( $event->Moving ) {
+		my $doc = $self->{Document};
+		if ( $doc->can('event_mouse_moving') ) {
+			$doc->event_mouse_moving( $self, $event );
+		}
+	} else {
+
+		# For a drag event...
+	}
+
+	$event->Skip;
+
+}
+
 sub on_middle_up {
 	my ( $self, $event ) = @_;
 
-	# TODO: Sometimes there are unexpected effects when using the middle button.
+	my $config = $self->main->ide->config;
+
+	# TO DO: Sometimes there are unexpected effects when using the middle button.
 	# It seems that another event is doing something but not within this module.
 	# Please look at ticket #390 for details!
 
-	Padre::Current->editor->Paste;
+	Wx::wxTheClipboard->UsePrimarySelection(1)
+		if $config->mid_button_paste;
+
+	if ( Padre::Constant::WIN32 or ( !$config->mid_button_paste ) ) {
+		Padre::Current->editor->Paste;
+	}
 
 	my $doc = $self->{Document};
 	if ( $doc->can('event_on_middle_up') ) {
 		$doc->event_on_middle_up( $self, $event );
 	}
 
-	$event->Skip;
+	Wx::wxTheClipboard->UsePrimarySelection(0)
+		if $config->mid_button_paste;
+
+	if ( $config->mid_button_paste ) {
+		$event->Skip;
+	} else {
+		$event->Skip(0);
+	}
 	return;
 }
 
+
 sub on_right_down {
 	my $self  = shift;
 	my $event = shift;
@@ -844,202 +1163,31 @@ sub on_right_down {
 	#my $p = $event->GetLogicalPosition;
 	#print "x: ", $p->x, "\n";
 
-	my $menu = Wx::Menu->new;
-	my $undo = $menu->Append( Wx::wxID_UNDO, '' );
-	if ( not $self->CanUndo ) {
-		$undo->Enable(0);
-	}
-	my $z = Wx::Event::EVT_MENU(
-		$main, # Ctrl-Z
-		$undo,
-		sub {
-			my $editor = Padre::Current->editor;
-			if ( $editor->CanUndo ) {
-				$editor->Undo;
-			}
-			return;
-		},
-	);
-	my $redo = $menu->Append( Wx::wxID_REDO, '' );
-	if ( not $self->CanRedo ) {
-		$redo->Enable(0);
-	}
-
-	Wx::Event::EVT_MENU(
-		$main, # Ctrl-Y
-		$redo,
-		sub {
-			my $editor = Padre::Current->editor;
-			if ( $editor->CanRedo ) {
-				$editor->Redo;
-			}
-			return;
-		},
-	);
-	$menu->AppendSeparator;
-
-	my $selection_exists = 0;
-	my $id               = $main->notebook->GetSelection;
-	if ( $id != -1 ) {
-		my $text = $main->notebook->GetPage($id)->GetSelectedText;
-		if ( defined($text) && length($text) > 0 ) {
-			$selection_exists = 1;
-		}
-	}
-
-	my $sel_all = $menu->Append( Wx::wxID_SELECTALL, Wx::gettext("Select all\tCtrl-A") );
-	if ( not $main->notebook->GetPage($id)->GetTextLength > 0 ) {
-		$sel_all->Enable(0);
-	}
-	Wx::Event::EVT_MENU(
-		$main, # Ctrl-A
-		$sel_all,
-		sub { \&text_select_all(@_) },
-	);
-	$menu->AppendSeparator;
-
-	my $copy = $menu->Append( Wx::wxID_COPY, Wx::gettext("&Copy\tCtrl-C") );
-	if ( not $selection_exists ) {
-		$copy->Enable(0);
-	}
-	Wx::Event::EVT_MENU(
-		$main, # Ctrl-C
-		$copy,
-		sub {
-			Padre::Current->editor->Copy;
-		}
-	);
-
-	my $cut = $menu->Append( Wx::wxID_CUT, Wx::gettext("Cu&t\tCtrl-X") );
-	if ( not $selection_exists ) {
-		$cut->Enable(0);
-	}
-	Wx::Event::EVT_MENU(
-		$main, # Ctrl-X
-		$cut,
-		sub {
-			Padre::Current->editor->Cut;
-		}
-	);
-
-	my $paste = $menu->Append( Wx::wxID_PASTE, Wx::gettext("&Paste\tCtrl-V") );
-	my $text = $self->get_text_from_clipboard();
-
-	if ( defined($text) and length($text) && $main->notebook->GetPage($id)->CanPaste ) {
-		Wx::Event::EVT_MENU(
-			$main, # Ctrl-V
-			$paste,
-			sub {
-				Padre::Current->editor->Paste;
-			},
-		);
-	} else {
-		$paste->Enable(0);
-	}
-
-	$menu->AppendSeparator;
-
-	my $commentToggle = $menu->Append( -1, Wx::gettext("&Toggle Comment\tCtrl-Shift-C") );
-	Wx::Event::EVT_MENU(
-		$main,
-		$commentToggle,
-		sub {
-			Padre::Wx::Main::on_comment_block( $_[0], 'TOGGLE' );
-		},
-	);
-	my $comment = $menu->Append( -1, Wx::gettext("&Comment Selected Lines\tCtrl-M") );
-	Wx::Event::EVT_MENU(
-		$main, $comment,
-		sub {
-			Padre::Wx::Main::on_comment_block( $_[0], 'COMMENT' );
-		},
-	);
-	my $uncomment = $menu->Append( -1, Wx::gettext("&Uncomment Selected Lines\tCtrl-Shift-M") );
-	Wx::Event::EVT_MENU(
-		$main,
-		$uncomment,
-		sub {
-			Padre::Wx::Main::on_comment_block( $_[0], 'UNCOMMENT' );
-		},
-	);
-
-	if (    $event->isa('Wx::MouseEvent')
-		and $self->main->ide->config->editor_folding )
-	{
-		$menu->AppendSeparator;
-
-		my $mousePos         = $event->GetPosition;
-		my $line             = $self->LineFromPosition( $self->PositionFromPoint($mousePos) );
-		my $firstPointInLine = $self->PointFromPosition( $self->PositionFromLine($line) );
-
-		if (   $mousePos->x < $firstPointInLine->x
-			&& $mousePos->x > ( $firstPointInLine->x - 18 ) )
-		{
-			my $fold = $menu->Append( -1, Wx::gettext("Fold all") );
-			Wx::Event::EVT_MENU(
-				$main, $fold,
-				sub {
-					$_[0]->current->editor->fold_all;
-				},
-			);
-			my $unfold = $menu->Append( -1, Wx::gettext("Unfold all") );
-			Wx::Event::EVT_MENU(
-				$main, $unfold,
-				sub {
-					$_[0]->current->editor->unfold_all;
-				},
-			);
-			$menu->AppendSeparator;
-		}
-	}
-
-	my $doc = $self->{Document};
-	if ( $doc->can('event_on_right_down') ) {
-		$doc->event_on_right_down( $self, $menu, $event );
-	}
-
-	# Let the plugins have a go
-	$self->main->ide->plugin_manager->on_context_menu( $doc, $self, $menu, $event );
+	require Padre::Wx::Menu::RightClick;
+	my $menu = Padre::Wx::Menu::RightClick->new( $main, $self, $event );
 
 	if ( $event->isa('Wx::MouseEvent') ) {
-		$self->PopupMenu( $menu, $event->GetX, $event->GetY );
+		$self->PopupMenu( $menu->wx, $event->GetX, $event->GetY );
 	} else { #Wx::CommandEvent
-		$self->PopupMenu( $menu, 50, 50 ); # TODO better location
+		$self->PopupMenu( $menu->wx, 50, 50 ); # TO DO better location
 	}
 }
 
-sub on_mouse_motion {
-	my $self   = shift;
-	my $event  = shift;
-	my $config = $self->main->ide->config;
 
-	$event->Skip;
-	return unless $config->main_syntaxcheck;
+# Convert the Ctrl-Scroll behaviour of changing the font size
+# to the non-Ctrl behaviour of scrolling.
+sub on_mousewheel {
+	my $self  = shift;
+	my $event = shift;
 
-	my $mousePos         = $event->GetPosition;
-	my $line             = $self->LineFromPosition( $self->PositionFromPoint($mousePos) );
-	my $firstPointInLine = $self->PointFromPosition( $self->PositionFromLine($line) );
-
-	my ( $offset1, $offset2 ) = ( 0, 18 );
-	if ( $config->editor_folding ) {
-		$offset1 += 18;
-		$offset2 += 18;
+	# Behave normally if Ctrl isn't down
+	unless ( $event->ControlDown and not $self->config->feature_fontsize ) {
+		$event->Skip(1);
+		return;
 	}
 
-	if (    $mousePos->x < ( $firstPointInLine->x - $offset1 )
-		and $mousePos->x > ( $firstPointInLine->x - $offset2 ) )
-	{
-		unless ( $self->MarkerGet($line) ) {
-			$self->CallTipCancel;
-			return;
-		}
-		$self->CallTipShow(
-			$self->PositionFromLine($line),
-			$self->{synchk_calltips}->{$line}
-		);
-	} else {
-		$self->CallTipCancel;
-	}
+	# Behave as if Ctrl wasn't down
+	$self->ScrollLines( $event->GetLinesPerAction * int( $event->GetWheelRotation / $event->GetWheelDelta * -1 ) );
 
 	return;
 }
@@ -1192,17 +1340,23 @@ sub _convert_paste_eols {
 }
 
 sub put_text_to_clipboard {
-	my ( $self, $text ) = @_;
+	my ( $self, $text, $clipboard ) = @_;
 	@_ = (); # Feeble attempt to kill Scalars Leaked
 
 	return if $text eq '';
 
+	my $config = $self->main->ide->config;
+
+	$clipboard ||= 0;
+
 	# Backup last clipboard value:
 	$self->{Clipboard_Old} = $self->get_text_from_clipboard;
 
 	#         if $self->{Clipboard_Old} ne $self->get_text_from_clipboard;
 
 	Wx::wxTheClipboard->Open;
+	Wx::wxTheClipboard->UsePrimarySelection($clipboard)
+		if $config->mid_button_paste;
 	Wx::wxTheClipboard->SetData( Wx::TextDataObject->new($text) );
 	Wx::wxTheClipboard->Close;
 
@@ -1234,7 +1388,10 @@ sub get_text_from_clipboard {
 # uncommented lines).
 sub comment_toggle_lines {
 	my ( $self, $begin, $end, $str ) = @_;
-	if ( _get_line_by_number( $self, $begin ) =~ /\s*$str/ ) {
+
+	my $comment_start = ref $str eq 'ARRAY' ? $str->[0] : $str;
+
+	if ( _get_line_by_number( $self, $begin ) =~ /^\s*\Q$comment_start\E/ ) {
 		uncomment_lines(@_);
 	} else {
 		comment_lines(@_);
@@ -1244,6 +1401,12 @@ sub comment_toggle_lines {
 # $editor->comment_lines($begin, $end, $str);
 # $str is either # for perl or // for Javascript, etc.
 # $str might be ['<--', '-->] for html
+#
+# Change: for Single lines comments, it will (un)comment with indent:
+# <indent>$comment_characters<space>XXXXXXX
+# If someone has idee for commenting Haskell Guards in Single lines,
+# (well, ('-- |') is a symbol for haddock.) please fix it.
+#
 sub comment_lines {
 	my ( $self, $begin, $end, $str ) = @_;
 
@@ -1254,16 +1417,18 @@ sub comment_lines {
 		$pos = $self->GetLineEndPosition($end);
 		$self->InsertText( $pos, $str->[1] );
 	} else {
-		my $is_first_column = $self->GetColumn( $self->GetCurrentPos ) == 0;
-		if ( $is_first_column && $end > $begin ) {
-			$end--;
-		}
-		for my $line ( $begin .. $end ) {
+		foreach my $line ( $begin .. $end ) {
+			my $text = _get_line_by_number( $self, $line );
 
-			# insert $str (# or //)
-			my $pos = $self->PositionFromLine($line);
-			$self->InsertText( $pos, $str );
+			# next if (length($text) == 0);  # should i do this?
+
+			if ( $text =~ /^(\s*)/ ) {
+				my $pos = $self->PositionFromLine($line);
+				$pos += length($1);
+				$self->InsertText( $pos, $str . ' ' );
+			}
 		}
+
 	}
 	$self->EndUndoAction;
 	return;
@@ -1273,6 +1438,7 @@ sub comment_lines {
 # $editor->uncomment_lines($begin, $end, $str);
 #
 # uncomment lines $begin..$end
+# Change: see comments for `comment_lines()`
 #
 sub uncomment_lines {
 	my ( $self, $begin, $end, $str ) = @_;
@@ -1294,17 +1460,16 @@ sub uncomment_lines {
 			$self->ReplaceSelection('');
 		}
 	} else {
-		my $length          = length $str;
-		my $is_first_column = $self->GetColumn( $self->GetCurrentPos ) == 0;
-		if ( $is_first_column && $end > $begin ) {
-			$end--;
-		}
-		for my $line ( $begin .. $end ) {
-			my $first = $self->PositionFromLine($line);
-			my $last  = $first + $length;
-			my $text  = $self->GetTextRange( $first, $last );
-			if ( $text eq $str ) {
-				$self->SetSelection( $first, $last );
+		foreach my $line ( $begin .. $end ) {
+			my $text = _get_line_by_number( $self, $line );
+
+			# the first line starting with '#!' can't be uncommented!
+			next if ( $line == 0 && $text =~ /^#!/ );
+
+			if ( $text =~ /^(\s*)(\Q$str\E\s*)/ ) {
+				my $start = $self->PositionFromLine($line) + length($1);
+
+				$self->SetSelection( $start, $start + length($2) );
 				$self->ReplaceSelection('');
 			}
 		}
@@ -1350,11 +1515,18 @@ sub configure_editor {
 
 	$doc->{newline_type} = $newline_type;
 
+	# Set the cursor blink rate
+	$self->SetCaretPeriod( $self->main->config->editor_cursor_blink );
+
+
 	return;
 }
 
 sub goto_line_centerize {
-	$_[0]->goto_pos_centerize( $_[0]->PositionFromLine( $_[1] ) );
+	my $self = shift;
+	my $line = shift;
+
+	$self->goto_pos_centerize( $self->PositionFromLine($line) );
 }
 
 # borrowed from Kephra
@@ -1399,6 +1571,7 @@ sub insert_from_file {
 		binmode($fh);
 		local $/ = undef;
 		$text = <$fh>;
+		close $fh;
 	} else {
 		return;
 	}
@@ -1484,7 +1657,7 @@ sub needs_manual_colorize {
 
 1;
 
-# Copyright 2008-2009 The Padre development team as listed in Padre.pm.
+# Copyright 2008-2010 The Padre development team as listed in Padre.pm.
 # LICENSE
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl 5 itself.
