@@ -5,16 +5,15 @@ package Padre::Wx::Menu::Refactor;
 use 5.008;
 use strict;
 use warnings;
-use List::Util    ();
-use File::Spec    ();
-use File::HomeDir ();
-use Params::Util qw{_INSTANCE};
+use List::Util      ();
+use File::Spec      ();
+use File::HomeDir   ();
 use Padre::Wx       ();
 use Padre::Wx::Menu ();
 use Padre::Locale   ();
-use Padre::Current qw{_CURRENT};
+use Padre::Current  ();
 
-our $VERSION = '0.63';
+our $VERSION = '0.76';
 our @ISA     = 'Padre::Wx::Menu';
 
 
@@ -37,21 +36,55 @@ sub new {
 	# Cache the configuration
 	$self->{config} = Padre->ide->config;
 
-
 	# Perl-Specific Refactoring
 	$self->{rename_variable} = $self->add_menu_action(
 		$self,
-		'refactor.rename_variable',
+		'perl.rename_variable',
+	);
+
+	# Create the variable-style submenu
+	my $style = Wx::Menu->new;
+	$self->{variable_style_menu} = $self->Append(
+		-1,
+		Wx::gettext('Change variable style'),
+		$style,
+	);
+
+	$self->add_menu_action(
+		$style,
+		'perl.variable_to_camel_case',
+	);
+
+	$self->add_menu_action(
+		$style,
+		'perl.variable_to_camel_case_ucfirst',
+	);
+
+	$self->add_menu_action(
+		$style,
+		'perl.variable_from_camel_case',
+	);
+
+	$self->add_menu_action(
+		$style,
+		'perl.variable_from_camel_case_ucfirst',
 	);
 
 	$self->{extract_subroutine} = $self->add_menu_action(
 		$self,
-		'refactor.extract_subroutine',
+		'perl.extract_subroutine',
 	);
 
 	$self->{introduce_temporary} = $self->add_menu_action(
 		$self,
-		'refactor.introduce_temporary',
+		'perl.introduce_temporary',
+	);
+
+	$self->AppendSeparator;
+
+	$self->{endify_pod} = $self->add_menu_action(
+		$self,
+		'perl.endify_pod',
 	);
 
 	return $self;
@@ -63,13 +96,14 @@ sub title {
 
 sub refresh {
 	my $self     = shift;
-	my $current  = _CURRENT(@_);
-	my $config   = $current->config;
+	my $current  = Padre::Current::_CURRENT(@_);
 	my $document = $current->document;
 
-	$self->{rename_variable}->Enable( $document->can('lexical_variable_replacement')     ? 1 : 0 );
+	$self->{rename_variable}->Enable( $document->can('rename_variable')                  ? 1 : 0 );
 	$self->{introduce_temporary}->Enable( $document->can('introduce_temporary_variable') ? 1 : 0 );
 	$self->{extract_subroutine}->Enable( $document->can('extract_subroutine')            ? 1 : 0 );
+	$self->{endify_pod}->Enable( $document->isa('Padre::Document::Perl')                 ? 1 : 0 );
+	$self->{variable_style_menu}->Enable( $document->isa('Padre::Document::Perl')        ? 1 : 0 );
 
 	return;
 }
